@@ -11,7 +11,7 @@
 #include "util/arg_auto_set.h"
 #include "Rev.h"
 
-#define version "3.1.0"
+#define version "3.2.0"
 
 #define LIBRND_ROOT         "../src/librnd"
 #define LIBRND_PLUGIN_ROOT  "../src/librnd/plugins"
@@ -24,6 +24,9 @@
 /* this turns off some code that is required for applications only */
 #define LIBRNDS_SCCONFIG 1
 
+/* we are doing /local/librnd/ */
+#define LIBRND_SCCONFIG_APP_TREE "librnd"
+
 #include "librnd/scconfig/plugin_3state.h"
 #include "librnd/scconfig/hooks_common.h"
 #include "librnd/scconfig/rnd_hook_detect.h"
@@ -71,13 +74,20 @@ int hook_custom_arg(const char *key, const char *value)
 			exit(1);
 		}
 		put("/local/pcb/coord_bits", value);
+		put("/local/librnd/coord_bits", value);
 		want_coord_bits = v;
 		return 1;
 	}
 	if (strncmp(key, "workaround-", 11) == 0) {
 		const char *what = key+11;
-		if (strcmp(what, "gtk-ctrl") == 0) append("/local/pcb/workaround_defs", "\n#define RND_WORKAROUND_GTK_CTRL 1");
-		else if (strcmp(what, "gtk-shift") == 0) append("/local/pcb/workaround_defs", "\n#define RND_WORKAROUND_GTK_SHIFT 1");
+		if (strcmp(what, "gtk-ctrl") == 0) {
+			append("/local/pcb/workaround_defs", "\n#define RND_WORKAROUND_GTK_CTRL 1");
+			append("/local/librnd/workaround_defs", "\n#define RND_WORKAROUND_GTK_CTRL 1");
+		}
+		else if (strcmp(what, "gtk-shift") == 0) {
+			append("/local/pcb/workaround_defs", "\n#define RND_WORKAROUND_GTK_SHIFT 1");
+			append("/local/librnd/workaround_defs", "\n#define RND_WORKAROUND_GTK_SHIFT 1");
+		}
 		else {
 			report("ERROR: unknown workaround '%s'\n", what);
 			exit(1);
@@ -86,12 +96,15 @@ int hook_custom_arg(const char *key, const char *value)
 	}
 	if (strcmp(key, "disable-so") == 0) {
 		put("/local/pcb/disable_so", strue);
+		put("/local/librnd/disable_so", strue);
 		put("/local/pcb/want_static_librnd", strue);
+		put("/local/librnd/want_static_librnd", strue);
 		pup_set_debug(strue);
 		return 1;
 	}
 	if (strcmp(key, "static-librnd") == 0) {
 		put("/local/pcb/want_static_librnd", strue);
+		put("/local/librnd/want_static_librnd", strue);
 		pup_set_debug(strue);
 		return 1;
 	}
@@ -116,10 +129,12 @@ int hook_postinit()
 {
 	db_mkdir("/local");
 	db_mkdir("/local/pcb");
+	db_mkdir("/local/librnd");
 
 	rnd_hook_postinit();
 
 	put("/local/pcb/coord_bits", "32");
+	put("/local/librnd/coord_bits", "32");
 	want_coord_bits = 32;
 
 	put("/local/librnd/no_confdir", strue);
@@ -143,11 +158,16 @@ int hook_detect_host()
 
 static void rnd_hook_detect_hid()
 {
-	int need_gtklibs = 0, want_gtk, want_gtk2, has_gtk2 = 0, want_gtk4, has_gtk4 = 0, want_gl, want_glib = 0;
+	int want_gtk2, has_gtk2 = 0, expl_gtk2, want_gtk4, has_gtk4 = 0, expl_gtk4;
+	int need_gtklibs = 0, want_gtk, want_gl, want_glib = 0, expl_gtk2_bi, expl_gtk4_bi;
 
 	want_gtk2   = plug_is_enabled("hid_gtk2_gdk") || plug_is_enabled("hid_gtk2_gl");
 	want_gtk4   = plug_is_enabled("hid_gtk4_gl");
 	want_gtk    = want_gtk2 | want_gtk4;
+	expl_gtk2   = plug_is_explicit("hid_gtk2_gdk") || plug_is_explicit("hid_gtk2_gl");
+	expl_gtk4   = plug_is_explicit("hid_gtk4_gl");
+	expl_gtk2_bi = plug_is_explicit_buildin("hid_gtk2_gdk") || plug_is_explicit_buildin("hid_gtk2_gl");
+	expl_gtk4_bi = plug_is_explicit_buildin("hid_gtk4_gl");
 
 	if (want_gtk2) {
 		require("libs/gui/gtk2/presents", 0, 0);
@@ -155,7 +175,7 @@ static void rnd_hook_detect_hid()
 			require("libs/gui/gtk2/key_prefix", 0, 1);
 			require("libs/gui/gtk2gl/presents", 0, 0);
 			if (!istrue(get("libs/gui/gtk2gl/presents"))) {
-				report_repeat("WARNING: Since there's no gl support for gtk found, disabling the gl rendering...\n");
+				report_repeat("WARNING: Since there's no gl support for gtk2 found, disabling the hid_gtk2_gl rendering... (you may need to install gtkglext1)\n");
 				hook_custom_arg("disable-hid_gtk2_gl", NULL);
 			}
 			need_gtklibs = 1;
@@ -190,8 +210,12 @@ static void rnd_hook_detect_hid()
 			report_repeat("WARNING: Since there's no GLU found, disabling the hid_gtk*_gl plugin...\n");
 			goto disable_gl;
 		}
-		else
+		else {
 			put("/local/pcb/has_glu", strue);
+			put("/local/librnd/has_glu", strue);
+		}
+		require("libs/gui/gl/vao/presents", 0, 0);
+		require("libs/gui/gl/fb_attachment/presents", 0, 0);
 	}
 	else {
 		disable_gl:;
@@ -201,31 +225,53 @@ static void rnd_hook_detect_hid()
 	}
 
 	if (has_gtk2 && has_gtk4 && (plug_is_buildin("lib_gtk2_common") || plug_is_buildin("lib_gtk4_common"))) {
-		if (plug_is_buildin("lib_gtk2_common")) {
+		if (expl_gtk2 && expl_gtk4) {
+			report_repeat("WARNING: you explicitly requested both gtk2 and gtk4, at least one of them as builtin, which is impossible. Converting them both to plugin.\n");
+
+			if (plug_is_enabled("hid_gtk2_gdk"))
+				hook_custom_arg("plugin-hid_gtk2_gdk", NULL);
+
+			if (plug_is_enabled("hid_gtk2_gl"))
+				hook_custom_arg("plugin-hid_gtk2_gl", NULL);
+
+			if (plug_is_enabled("hid_gtk4_gl"))
+				hook_custom_arg("plugin-hid_gtk4_gl", NULL);
+
+			hook_custom_arg("plugin-lib_gtk2_common", NULL);
+			hook_custom_arg("plugin-lib_gtk4_common", NULL);
+		}
+		else if (!expl_gtk2 && expl_gtk4) {
+			if (plug_is_buildin("lib_gtk4_common"))
+				report_repeat("Note: you explicitly requested gtk4 as builtin; gtk2 would be available, but if gtk4 is builtin, I have to disable gtk2. If you need both, request both to be plugins.\n");
+			else
+				report_repeat("Note: you explicitly requested gtk4 as plugin; gtk2 would be available, but can not be a builtin (default preference), so I have to disable gtk2. If you need both, request both to be plugins.\n");
+			goto choose_gtk4;
+		}
+		else if (expl_gtk2 && !expl_gtk4) {
+			if (plug_is_buildin("lib_gtk2_common"))
+				report_repeat("Note: you explicitly requested gtk2 as builtin; gtk4 would be available, but if gtk2 is builtin, I have to disable gtk4. If you need both, request both gtk4 and gtk2 to be plugins.\n");
+			else
+				report_repeat("Note: you explicitly requested gtk2 as plugin; gtk4 would be available, but can not be a builtin, so I have to disable gtk4. If you need both, request both to be plugins.\n");
+			goto choose_gtk2;
+		}
+		else if (plug_is_buildin("lib_gtk2_common")) { /* implicit, prefer gtk2 */
+			report_repeat("WARNING: you have both gtk2 and gtk4 installed; you can't have any gtk built-in and use the other as well. Since gtk2 is builtin, disabling gtk4.\n");
+			choose_gtk2:;
 			hook_custom_arg("disable-hid_gtk4_gl", NULL);
 			hook_custom_arg("disable-lib_gtk4_common", NULL);
-			report_repeat("WARNING: you have both gtk2 and gtk4 installed; you can't have any gtk built-in and use the other as well. Since gtk2 is builtin, disabling gtk4.\n");
 		}
-		else if (plug_is_buildin("lib_gtk4_common")) {
+		else if (plug_is_buildin("lib_gtk4_common")) { /* implicit, prefer gtk4 */
+			report_repeat("WARNING: you have both gtk2 and gtk4 installed; you can't have any gtk built-in and use the other as well. Since gtk4 is builtin, disabling gtk2.\n");
+			choose_gtk4:;
 			hook_custom_arg("disable-hid_gtk2_gl", NULL);
 			hook_custom_arg("disable-lib_gtk2_common", NULL);
-			report_repeat("WARNING: you have both gtk2 and gtk4 installed; you can't have any gtk built-in and use the other as well. Since gtk4 is builtin, disabling gtk2.\n");
 		}
 	}
 
-	/* libs/gui/gtkx is the version-independent set of flags in the XOR model */
-	if (has_gtk2) {
-		put("/target/libs/gui/gtkx/cflags", get("/target/libs/gui/gtk2/cflags"));
-		put("/target/libs/gui/gtkx/ldflags", get("/target/libs/gui/gtk2/ldflags"));
-	}
-	else if (has_gtk4) {
-		put("/target/libs/gui/gtkx/cflags", get("/target/libs/gui/gtk4/cflags"));
-		put("/target/libs/gui/gtkx/ldflags", get("/target/libs/gui/gtk4/ldflags"));
-	}
-
 	if (!need_gtklibs) {
-		report("No gtk support available, disabling lib_gtk_common...\n");
-		hook_custom_arg("disable-lib_gtk_common", NULL);
+		report("No gtk support available, disabling lib_gtk*_common...\n");
+		hook_custom_arg("disable-lib_gtk2_common", NULL);
+		hook_custom_arg("disable-lib_gtk4_common", NULL);
 	}
 
 	if (plug_is_enabled("hid_lesstif")) {
@@ -236,7 +282,8 @@ static void rnd_hook_detect_hid()
 			require("libs/gui/xrender/presents", 0, 0);
 		}
 		else {
-			report_repeat("WARNING: Since there's no usable lesstif2 found, disabling the lesstif HID and xinerama and xrender...\n");
+			if (plug_is_explicit("hid_lesstif"))
+				report_repeat("WARNING: Since there's no usable lesstif2 found, disabling the lesstif HID and xinerama and xrender...\n");
 			hook_custom_arg("disable-xinerama", NULL);
 			hook_custom_arg("disable-xrender", NULL);
 			hook_custom_arg("disable-hid_lesstif", NULL);
@@ -286,10 +333,12 @@ static void rnd_hook_detect_coord_bits(void)
 
 	sprintf(tmp, "((1%s<<%d)-1)", postfix, want_coord_bits - 1);
 	put("/local/pcb/coord_type", chosen);
+	put("/local/librnd/coord_type", chosen);
 	put("/local/pcb/coord_max", tmp);
+	put("/local/librnd/coord_max", tmp);
 
 	chosen = NULL;
-	if (istrue(get("/local/pcb/debug"))) { /* debug: c89 */
+	if (istrue(get("/local/librnd/debug"))) { /* debug: c89 */
 		if (int64_bits >= 64) {
 			/* to suppress warnings on systems that support c99 but are forced to compile in c89 mode */
 			chosen = "int64_t";
@@ -303,8 +352,11 @@ static void rnd_hook_detect_coord_bits(void)
 		else chosen = "double";
 	}
 	put("/local/pcb/long64", chosen);
-	if (need_stdint)
+	put("/local/librnd/long64", chosen);
+	if (need_stdint) {
 		put("/local/pcb/include_stdint", "#include <stdint.h>");
+		put("/local/librnd/include_stdint", "#include <stdint.h>");
+	}
 }
 
 
@@ -314,7 +366,8 @@ static void rnd_hook_detect_coord_bits(void)
 static const char *dialog_deps[] = {
 	"/local/pcb/dialogs/controls",         /* so we don't relax user's explicit request */
 	"/local/pcb/hid_remote/controls",
-	"/local/pcb/lib_gtk_common/controls",
+	"/local/pcb/lib_gtk2_common/controls",
+	"/local/pcb/lib_gtk4_common/controls",
 	"/local/pcb/hid_lesstif/controls",
 	NULL
 };
@@ -345,7 +398,7 @@ int hook_detect_target()
 {
 	int want_gd, want_stroke;
 
-	want_gd     = plug_is_enabled("export_png") || plug_is_enabled("import_pxm_gd");
+	want_gd     = plug_is_enabled("lib_exp_pixmap") || plug_is_enabled("import_pixmap_gd");
 	want_stroke = plug_is_enabled("stroke");
 
 	rnd_hook_detect_cc();
@@ -355,7 +408,8 @@ int hook_detect_target()
 	if (want_stroke) {
 		require("libs/gui/libstroke/presents", 0, 0);
 		if (!istrue(get("libs/gui/libstroke/presents"))) {
-			report_repeat("WARNING: Since there's no libstroke found, disabling the stroke plugin...\n");
+			if (plug_is_explicit("stroke"))
+				report_repeat("WARNING: Since there's no libstroke found, disabling the stroke plugin...\n");
 			hook_custom_arg("disable-stroke", NULL);
 		}
 	}
@@ -370,10 +424,8 @@ int hook_detect_target()
 			hook_custom_arg("disable-gd-gif", NULL);
 			hook_custom_arg("disable-gd-png", NULL);
 			hook_custom_arg("disable-gd-jpg", NULL);
-#ifdef PNG
-			hook_custom_arg("disable-export_png", NULL);
-			hook_custom_arg("disable-import_pxm_gd", NULL);
-#endif
+			hook_custom_arg("disable-lib_exp_pixmap", NULL);
+			hook_custom_arg("disable-import_pixmap_gd", NULL);
 			want_gd = 0;
 			goto disable_gd_formats;
 		}
@@ -429,6 +481,7 @@ static int rnd_hook_generate()
 	printf("Generating compat_inc.h (%d)\n", generr |= tmpasm(LIBRND_ROOT "/core", "compat_inc.h.in", "compat_inc.h"));
 	printf("Generating librnd config.h (%d)\n", generr |= tmpasm(LIBRND_ROOT, "config.h.in", "config.h"));
 	printf("Generating opengl.h (%d)\n", generr |= tmpasm(LIBRND_PLUGIN_ROOT "/lib_hid_gl", "opengl.h.in", "opengl.h"));
+	printf("Generating draw_INIT.h (%d)\n", generr |= tmpasm(LIBRND_PLUGIN_ROOT "/lib_hid_gl", "draw_INIT.h.in", "draw_INIT.h"));
 
 	return generr;
 }
@@ -497,9 +550,9 @@ int hook_generate()
 	printf("Configuration summary\n");
 	printf("=====================\n");
 
-	print_sum_setting("/local/pcb/debug",          "Compilation for debugging");
-	print_sum_setting_or("/local/pcb/symbols",     "Include debug symbols", istrue(get("/local/pcb/debug")));
-	print_sum_cfg_val("/local/pcb/coord_bits",     "Coordinate type bits");
+	print_sum_setting("/local/librnd/debug",          "Compilation for debugging");
+	print_sum_setting_or("/local/librnd/symbols",     "Include debug symbols", istrue(get("/local/pcb/debug")));
+	print_sum_cfg_val("/local/librnd/coord_bits",     "Coordinate type bits");
 
 #undef plugin_def
 #undef plugin_header
