@@ -48,9 +48,8 @@
 #include <librnd/core/hidlib.h>
 #include <librnd/core/fptr_cast.h>
 #include <librnd/core/safe_fs_dir.h>
+#include <librnd/core/event.h>
 
-/* conf list node's name */
-const char *rnd_conf_list_name = "pcb-rnd-conf-v1";
 static const char *flcat = "conf";
 
 static const char conf_cookie_al[] = "core/conf/anyload";
@@ -63,6 +62,30 @@ extern void rnd_pcbhl_conf_postproc(void);
 
 int rnd_conf_in_production = 0;
 
+/* conf list node's name */
+static const char *rnd_conf_list_name(void)
+{
+	static char tmp[80];
+	static int inited = 0;
+
+	if (!inited) {
+		int len = strlen(rnd_app.package);
+		if (len > sizeof(tmp)-10) {
+			fprintf(stderr, "librnd internal error: rnd_app.package '%s' is too long\n", rnd_app.package);
+			exit(1);
+		}
+		memcpy(tmp, rnd_app.package, len);
+		strcpy(tmp+len, "-conf-v1");
+		inited = 1;
+	}
+	return tmp;
+}
+
+static int rnd_conf_list_match(const char *str)
+{
+	return (strcmp(str, rnd_conf_list_name()) == 0) || (strcmp(str, "pcb-rnd-conf-v1") == 0);
+}
+
 /* The main conf: monolithic config files affecting all parts of the conf tree;
    By default every operation is done on these trees. */
 lht_doc_t *rnd_conf_main_root[RND_CFR_max_alloc];
@@ -97,21 +120,21 @@ static lht_node_t *conf_lht_get_confroot(lht_node_t *cwd)
 		return NULL;
 
 	/* if it's a list with a matching name, we found it */
-	if ((cwd->type == LHT_LIST) && (strcmp(cwd->name, rnd_conf_list_name) == 0))
+	if ((cwd->type == LHT_LIST) && rnd_conf_list_match(cwd->name))
 		return cwd;
 
 	/* else it may be the parent-hash of the list */
 	if (cwd->type != LHT_HASH)
 		return NULL;
 
-	return lht_dom_hash_get(cwd, rnd_conf_list_name);
+	return lht_dom_hash_get(cwd, rnd_conf_list_name());
 }
 
 int rnd_conf_insert_tree_as(rnd_conf_role_t role, lht_node_t *root)
 {
 	lht_doc_t *d;
 
-	if ((root->type != LHT_LIST) || (strcmp(root->name, "pcb-rnd-conf-v1") != 0))
+	if ((root->type != LHT_LIST) || !rnd_conf_list_match(root->name))
 		return -1;
 
 	if (rnd_conf_main_root[role] != NULL) {
@@ -120,7 +143,7 @@ int rnd_conf_insert_tree_as(rnd_conf_role_t role, lht_node_t *root)
 	}
 
 	d = lht_dom_init();
-	d->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
+	d->root = lht_dom_node_alloc(LHT_LIST, rnd_conf_list_name());
 	d->root->doc = d;
 	lht_tree_merge(d->root, root);
 	rnd_conf_main_root[role] = d;
@@ -153,8 +176,8 @@ static lht_doc_t *conf_load_plug_file(const char *fn, int fn_is_text)
 		return NULL;
 	}
 
-	if ((d->root->type != LHT_LIST) || (strcmp(d->root->name, "pcb-rnd-conf-v1") != 0)) {
-		rnd_message(RND_MSG_ERROR, "error: failed to load lht plugin config: %s (not a pcb-rnd-conf-v1)\n", ifn);
+	if ((d->root->type != LHT_LIST) || !rnd_conf_list_match(d->root->name)) {
+		rnd_message(RND_MSG_ERROR, "error: failed to load lht plugin config: %s (not a %s)\n", ifn, rnd_conf_list_name());
 		lht_dom_uninit(d);
 		return NULL;
 	}
@@ -162,6 +185,18 @@ static lht_doc_t *conf_load_plug_file(const char *fn, int fn_is_text)
 	return d;
 }
 
+static lht_node_t *init_project_conf(lht_doc_t *d)
+{
+	lht_node_t *prjroot, *confroot;
+	prjroot = lht_dom_node_alloc(LHT_HASH, "coraleda-project-v1");
+	confroot = lht_dom_node_alloc(LHT_LIST, rnd_conf_list_name());
+	lht_dom_hash_put(prjroot, confroot);
+	prjroot->doc = d;
+	confroot->doc = d;
+	d->root = prjroot;
+	return confroot;
+}
+
 int rnd_conf_load_as(rnd_conf_role_t role, const char *fn, int fn_is_text)
 {
 	lht_doc_t *d;
@@ -180,7 +215,10 @@ int rnd_conf_load_as(rnd_conf_role_t role, const char *fn, int fn_is_text)
 		ifn = "<string>";
 	}
 	else {
-		d = rnd_hid_cfg_load_lht(NULL, fn);
+		if (fn != NULL)
+			d = rnd_hid_cfg_load_lht(NULL, fn);
+		else
+			d = lht_dom_init();
 		ifn = fn;
 	}
 
@@ -198,20 +236,14 @@ int rnd_conf_load_as(rnd_conf_role_t role, const char *fn, int fn_is_text)
 	}
 
 	if (d->root == NULL) {
-		lht_node_t *prjroot, *confroot;
-		prjroot = lht_dom_node_alloc(LHT_HASH, "coraleda-project-v1");
-		confroot = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
-		lht_dom_hash_put(prjroot, confroot);
-		prjroot->doc = d;
-		confroot->doc = d;
-		d->root = prjroot;
+		init_project_conf(d);
 		rnd_conf_main_root[role] = d;
 		if (role_name != NULL)
 			rnd_file_loaded_set_at(flcat, role_name, ifn, "project/conf");
 		return 0;
 	}
 
-	if ((d->root->type == LHT_LIST) && (strcmp(d->root->name, "pcb-rnd-conf-v1") == 0)) {
+	if ((d->root->type == LHT_LIST) && rnd_conf_list_match(d->root->name)) {
 		rnd_conf_main_root[role] = d;
 		if (role_name != NULL)
 			rnd_file_loaded_set_at(flcat, role_name, ifn, "conf");
@@ -220,24 +252,24 @@ int rnd_conf_load_as(rnd_conf_role_t role, const char *fn, int fn_is_text)
 
 	if ((d->root->type == LHT_HASH) && ((strcmp(d->root->name, "geda-project-v1") == 0) || (strcmp(d->root->name, "coraleda-project-v1") == 0))) {
 		lht_node_t *confroot;
-		confroot = lht_tree_path_(d, d->root, "pcb-rnd-conf-v1", 1, 0, NULL);
+		confroot = lht_tree_path_(d, d->root, rnd_conf_list_name(), 1, 0, NULL);
 
 		if (role_name != NULL)
 			rnd_file_loaded_set_at(flcat, role_name, ifn, "project/conf");
 
-		if ((confroot != NULL)  && (confroot->type == LHT_LIST) && (strcmp(confroot->name, "li:pcb-rnd-conf-v1") == 0)) {
+		if ((confroot != NULL) && (confroot->type == LHT_LIST) && (strncmp(confroot->name, "li:", 3) == 0) && rnd_conf_list_match(confroot->name+3)) {
 			rnd_conf_main_root[role] = d;
 			return 0;
 		}
 
 		/* project file with no config root */
-		confroot = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
+		confroot = lht_dom_node_alloc(LHT_LIST, rnd_conf_list_name());
 		lht_dom_hash_put(d->root, confroot);
 		rnd_conf_main_root[role] = d;
 		return 0;
 	}
 
-	rnd_hid_cfg_error(d->root, "Root node must be one of li:pcb-rnd-conf-v1, ha:coraleda-project-v1 or ha:geda-project-v1\n");
+	rnd_hid_cfg_error(d->root, "Root node must be one of li:%s-conf-v1, ha:coraleda-project-v1 or ha:geda-project-v1\n", rnd_app.package);
 
 	if (d != NULL)
 		lht_dom_uninit(d);
@@ -251,7 +283,7 @@ static int conf_merge_plug(lht_doc_t *d, rnd_conf_role_t role, const char *path)
 
 	if (rnd_conf_plug_root[role] == NULL) {
 		rnd_conf_plug_root[role] = lht_dom_init();
-		rnd_conf_plug_root[role]->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
+		rnd_conf_plug_root[role]->root = lht_dom_node_alloc(LHT_LIST, rnd_conf_list_name());
 		rnd_conf_plug_root[role]->root->doc = rnd_conf_plug_root[role];
 	}
 	err = lht_tree_merge(rnd_conf_plug_root[role]->root, d->root);
@@ -454,7 +486,12 @@ void rnd_conf_extract_poliprio(lht_node_t *root, rnd_conf_policy_t *gpolicy, lon
 int rnd_conf_get_policy_prio(lht_node_t *node, rnd_conf_policy_t *gpolicy, long *gprio)
 {
 	for(;;node = node->parent) {
-		if (node->parent == node->doc->root) {
+		int is_ppnode = 0;
+		if ((node->parent->parent == node->doc->root) || (node->parent == node->doc->root)) {
+			if (rnd_conf_list_match(node->parent->name))
+				is_ppnode = 1;
+		}
+		if (is_ppnode) {
 			rnd_conf_extract_poliprio(node, gpolicy, gprio);
 			return 0;
 		}
@@ -633,9 +670,9 @@ static void conf_insert_arr(rnd_conf_native_t *dest, int offs)
 			CASE_RND_MOVE(RND_CFN_HLIST, list);
 			case RND_CFN_max: break;
 		}
-	}
 #undef CASE_RND_MOVE
 	abort(); /* unhandled type */
+	}
 }
 
 int rnd_conf_merge_patch_array(rnd_conf_native_t *dest, lht_node_t *src_lst, int prio, rnd_conf_policy_t pol)
@@ -837,6 +874,11 @@ int rnd_conf_merge_patch_item(const char *path, lht_node_t *n, rnd_conf_role_t r
 	rnd_conf_native_t *target = rnd_conf_get_field(path);
 	int res = 0;
 
+	if (rnd_app.conf_dont_merge_node != NULL) {
+		if (rnd_app.conf_dont_merge_node(path, n, role, default_prio, default_policy, target))
+			return 0;
+	}
+
 	if ((role == RND_CFR_DESIGN) || (role == RND_CFR_PROJECT)) {
 		if (conf_board_ignore(path, n))
 			return 0;
@@ -1184,6 +1226,40 @@ lht_node_t *rnd_conf_lht_get_first_pol(rnd_conf_role_t target, rnd_conf_policy_t
 	return conf_lht_get_first_(rnd_conf_main_root[target]->root, pol, create, RND_POL_OVERWRITE);
 }
 
+lht_node_t *rnd_pref_ensure_conf_root(rnd_hidlib_t *hidlib, rnd_conf_role_t role)
+{
+	lht_node_t *m;
+
+	m = rnd_conf_lht_get_first(role, 0);
+	if (m == NULL) {
+		if (role == RND_CFR_PROJECT) {
+			const char *pcb_fn = (hidlib == NULL ? NULL : hidlib->filename);
+			const char *try, *fn = rnd_conf_get_project_conf_name(NULL, pcb_fn, &try);
+			if (fn == NULL) {
+				rnd_message(RND_MSG_ERROR, "Failed to create the project file\n");
+				return NULL;
+			}
+			rnd_conf_reset(role, fn);
+			rnd_conf_makedirty(role);
+			rnd_conf_save_file(hidlib, fn, pcb_fn, role, NULL);
+			m = rnd_conf_lht_get_first(role, 0);
+			if (m == NULL) {
+				rnd_message(RND_MSG_ERROR, "Failed to create the project file %s\n", fn);
+				return NULL;
+			}
+				rnd_message(RND_MSG_INFO, "Created the project file\n");
+		}
+		else {
+			rnd_message(RND_MSG_ERROR, "Failed to create config file for role %s\n", rnd_conf_role_name(role));
+			return NULL;
+		}
+	}
+
+	return m;
+}
+
+
+static const char empty_list_or_str[] = "";
 static lht_node_t *conf_lht_get_at_(rnd_conf_role_t target, const char *conf_path, const char *lht_path, int allow_plug, int create)
 {
 	lht_node_t *n, *r = NULL;
@@ -1194,7 +1270,7 @@ static lht_node_t *conf_lht_get_at_(rnd_conf_role_t target, const char *conf_pat
 
 	r = lht_tree_path_(n->doc, n, lht_path, 1, 0, NULL);
 	if ((r == NULL) && (create)) {
-		rnd_conf_set_dry(target, conf_path, -1, "", RND_POL_OVERWRITE, 0);
+		rnd_conf_set_dry(target, conf_path, -1, empty_list_or_str, RND_POL_OVERWRITE, 0);
 		r = lht_tree_path_(n->doc, n, lht_path, 1, 0, NULL);
 	}
 
@@ -1292,6 +1368,24 @@ void rnd_conf_load_project(const char *project_fn, const char *design_fn)
 	rnd_conf_update(NULL, -1);
 }
 
+
+static rnd_conf_native_t *rnd_conf_alloc_field_(void *value, int array_size, rnd_conf_native_type_t type, const char *path, const char *desc, rnd_conf_flag_t flags)
+{
+	rnd_conf_native_t *node = calloc(sizeof(rnd_conf_native_t), 1);
+
+	node->array_size  = array_size;
+	node->type        = type;
+	node->val.any     = value;
+	node->prop        = calloc(sizeof(rnd_confprop_t), array_size);
+	node->description = desc;
+	node->hash_path   = path;
+	node->flags       = flags;
+	vtp0_init(&(node->hid_data));
+	vtp0_init(&(node->hid_callbacks));
+
+	return node;
+}
+
 rnd_conf_native_t *rnd_conf_reg_field_(void *value, int array_size, rnd_conf_native_type_t type, const char *path, const char *desc, rnd_conf_flag_t flags)
 {
 	rnd_conf_native_t *node;
@@ -1303,17 +1397,7 @@ rnd_conf_native_t *rnd_conf_reg_field_(void *value, int array_size, rnd_conf_nat
 	assert(array_size >= 1);
 
 	assert(htsp_get(rnd_conf_fields, path) == NULL);
-
-	node = calloc(sizeof(rnd_conf_native_t), 1);
-	node->array_size  = array_size;
-	node->type        = type;
-	node->val.any     = value;
-	node->prop        = calloc(sizeof(rnd_confprop_t), array_size);
-	node->description = desc;
-	node->hash_path   = path;
-	node->flags       = flags;
-	vtp0_init(&(node->hid_data));
-	vtp0_init(&(node->hid_callbacks));
+	node = rnd_conf_alloc_field_(value, array_size, type, path, desc, flags);
 
 	htsp_set(rnd_conf_fields, (char *)path, node);
 	rnd_conf_hid_global_cb(node, -1, new_item_post);
@@ -1455,7 +1539,7 @@ int rnd_conf_set_dry(rnd_conf_role_t target, const char *path_, int arr_idx, con
 		return -1;
 	}
 
-	if (idx >= nat->array_size) {
+	if (((nat->type != RND_CFN_LIST) && (nat->type != RND_CFN_HLIST)) && (idx >= nat->array_size)) {
 		rnd_message(RND_MSG_ERROR, "Error: can't rnd_conf_set() %s[%d]: %d is beyond the end of the array (%d)\n", path, idx, idx, nat->array_size);
 		free(path);
 		return -1;
@@ -1531,7 +1615,7 @@ int rnd_conf_set_dry(rnd_conf_role_t target, const char *path_, int arr_idx, con
 	if (ty == LHT_LIST) {
 		lht_err_t err = 0;
 
-		if (new_val != NULL)
+		if ((new_val != NULL) && (new_val != empty_list_or_str))
 			nn = lht_dom_node_alloc(LHT_TEXT, "");
 
 		if (pol == RND_POL_OVERWRITE) {
@@ -1539,14 +1623,14 @@ int rnd_conf_set_dry(rnd_conf_role_t target, const char *path_, int arr_idx, con
 				/* empty the list so that we insert to an empty list which is overwriting the list */
 				while(cwd->data.list.first != NULL)
 					lht_tree_del(cwd->data.list.first);
-				if (new_val != NULL)
+				if ((new_val != NULL) && (new_val != empty_list_or_str))
 					lht_dom_list_append(cwd, nn);
 			}
 			else {
 				lht_node_t *old = lht_tree_list_nth(cwd, idx);
 				if (old != NULL) {
 					/* the list is large enough already: overwrite the element at idx */
-					if (new_val == NULL) {
+					if ((new_val == NULL) || (new_val == empty_list_or_str)) {
 						err = lht_tree_del(old);
 						free(path);
 						return 0;
@@ -1554,7 +1638,7 @@ int rnd_conf_set_dry(rnd_conf_role_t target, const char *path_, int arr_idx, con
 					else
 						err = lht_tree_list_replace_child(cwd, old, nn);
 				}
-				else if (new_val == NULL) {
+				else if ((new_val == NULL) || (new_val == empty_list_or_str)) {
 					free(path);
 					return 0;
 				}
@@ -1882,8 +1966,8 @@ int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char
 {
 	int fail = 1;
 	lht_node_t *r = rnd_conf_lht_get_first(role, 0);
-	const char *try;
-	char *efn;
+	const char *try, *event_fn = NULL;
+	char *efn = NULL;
 
 	/* do not save if there's no change */
 	if (rnd_conf_lht_dirty[role] == 0)
@@ -1909,6 +1993,8 @@ int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char
 		FILE *f;
 
 		f = rnd_fopen_fn(hidlib, fn, "w", &efn);
+		if (f != NULL)
+			event_fn = fn;
 		if ((f == NULL) && (role == RND_CFR_USER)) {
 			/* create the directory and try again */
 			char *path = NULL, *end;
@@ -1931,6 +2017,8 @@ int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char
 					rnd_message(RND_MSG_INFO, "Created directory %s for saving %s\n", path, fn);
 					*end = '/';
 					f = rnd_fopen(hidlib, path, "w");
+					if (f != NULL)
+						event_fn = path;
 				}
 				else
 					rnd_message(RND_MSG_ERROR, "Error: failed to create directory %s for saving %s\n", path, efn);
@@ -1948,6 +2036,8 @@ int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char
 			rnd_message(RND_MSG_ERROR, "Error: can't save config to %s - can't open the file for write\n", fn);
 	}
 
+	if (!fail)
+		rnd_event(hidlib, RND_EVENT_CONF_FILE_SAVE_POST, "si", event_fn, role);
 	free(efn);
 	return fail;
 }
@@ -2064,18 +2154,27 @@ rnd_conf_role_t rnd_conf_lookup_role(const lht_node_t *nd)
 
 void rnd_conf_reset(rnd_conf_role_t target, const char *source_fn)
 {
-	lht_node_t *n;
+	lht_node_t *n, *cfr;
 
 	if (rnd_conf_main_root[target] != NULL)
 		lht_dom_uninit(rnd_conf_main_root[target]);
 
 	rnd_conf_main_root[target] = lht_dom_init();
+
+
 	if (source_fn != NULL)
 		lht_dom_loc_newfile(rnd_conf_main_root[target], source_fn);
-	rnd_conf_main_root[target]->root = lht_dom_node_alloc(LHT_LIST, rnd_conf_list_name);
-	rnd_conf_main_root[target]->root->doc = rnd_conf_main_root[target];
+
+	if (target != RND_CFR_PROJECT) {
+		rnd_conf_main_root[target]->root = lht_dom_node_alloc(LHT_LIST, rnd_conf_list_name());
+		rnd_conf_main_root[target]->root->doc = rnd_conf_main_root[target];
+		cfr = rnd_conf_main_root[target]->root;
+	}
+	else
+		cfr = init_project_conf(rnd_conf_main_root[target]);
+
 	n = lht_dom_node_alloc(LHT_HASH, "overwrite");
-	lht_dom_list_insert(rnd_conf_main_root[target]->root, n);
+	lht_dom_list_insert(cfr, n);
 }
 
 /*****************/
@@ -2235,7 +2334,7 @@ void rnd_conf_uninit(void)
 
 	rnd_conf_fields_foreach(e) {
 		if (strncmp(e->key, "plugins/", 8) == 0)
-			fprintf(stderr, "pcb-rnd conf ERROR: conf node '%s' is not unregistered\n", e->key);
+			fprintf(stderr, "%s conf ERROR: conf node '%s' is not unregistered\n", rnd_app.package, e->key);
 	}
 
 	rnd_conf_hid_uninit();
@@ -2301,3 +2400,4 @@ void rnd_conf_ro(const char *path)
 }
 
 #include "conf_regfile.c"
+#include "conf_multi_temp.c"
