@@ -32,6 +32,7 @@
 
 #include <stdlib.h>
 #include <locale.h>
+#include <ctype.h>
 
 #ifdef __WIN32__
 #include <wchar.h>
@@ -64,10 +65,13 @@
 #include <librnd/core/grid.h>
 #include <librnd/core/funchash.h>
 #include <librnd/core/hid_menu.h>
+#include <librnd/core/compat_lrealpath.h>
+#include <librnd/core/safe_fs.h>
 #include "../../../config.h"
 
 int rnd_coord_t_size = sizeof(rnd_coord_t);
 
+const char *rnd_ver_str = RND_VER_STR;
 
 static const char *flt_any[] = {"*", "*.*", NULL};
 
@@ -106,11 +110,11 @@ void rnd_hid_init()
 
 	/* host app's */
 TODO("make this configurable - add to conf_board_ignores avoid plugin injection")
-	tmp = rnd_concat(rnd_conf.rc.path.exec_prefix, RND_DIR_SEPARATOR_S, "lib", RND_DIR_SEPARATOR_S, "pcb-rnd", RND_DIR_SEPARATOR_S, "plugins", RND_DIR_SEPARATOR_S, HOST, NULL);
+	tmp = rnd_concat(rnd_conf.rc.path.exec_prefix, RND_DIR_SEPARATOR_S, "lib", RND_DIR_SEPARATOR_S, rnd_app.package, RND_DIR_SEPARATOR_S, "plugins", RND_DIR_SEPARATOR_S, HOST, NULL);
 	rnd_plugin_add_dir(tmp);
 	free(tmp);
 
-	tmp = rnd_concat(rnd_conf.rc.path.exec_prefix, RND_DIR_SEPARATOR_S, "lib", RND_DIR_SEPARATOR_S, "pcb-rnd", RND_DIR_SEPARATOR_S, "plugins", NULL);
+	tmp = rnd_concat(rnd_conf.rc.path.exec_prefix, RND_DIR_SEPARATOR_S, "lib", RND_DIR_SEPARATOR_S, rnd_app.package, RND_DIR_SEPARATOR_S, "plugins", NULL);
 	rnd_plugin_add_dir(tmp);
 	free(tmp);
 
@@ -208,8 +212,7 @@ void rnd_hid_remove_hid(rnd_hid_t *hid)
 
 static void rnd_hid_load_gui_plugin(const char *plugname);
 
-/* Load all hid_* plugins from all puplug search paths */
-static void rnd_hid_load_all_gui_plugins(void)
+void rnd_hid_do_all_gui_plugins(void *ctx, int (*cb)(void *ctx, const char *name, const char *dir, const char *fn))
 {
 	char **dir;
 	gds_t tmp = {0};
@@ -236,10 +239,8 @@ static void rnd_hid_load_all_gui_plugins(void)
 				if (strcmp(s, ".pup") != 0)
 					continue;
 				*s = '\0';
-
-				if (rnd_conf.rc.verbose >= RND_MSG_INFO)
-					fprintf(stderr, "-- all-hid-load: '%s' ('%s' from %s)\n", tmp.array, fn, *dir);
-				rnd_hid_load_gui_plugin(tmp.array);
+				if (cb(ctx, tmp.array, *dir, fn) != 0)
+					break;
 			}
 		}
 
@@ -248,6 +249,59 @@ static void rnd_hid_load_all_gui_plugins(void)
 	gds_uninit(&tmp);
 }
 
+static int load_all_gui_plugins_cb(void *ctx, const char *name, const char *dir, const char *fn)
+{
+	if (rnd_conf.rc.verbose >= RND_MSG_INFO)
+		fprintf(stderr, "-- all-hid-load: '%s' ('%s' from %s)\n", name, fn, dir);
+	rnd_hid_load_gui_plugin(name);
+	return 0;
+}
+
+/* Load all hid_* plugins from all puplug search paths */
+static void rnd_hid_load_all_gui_plugins(void)
+{
+	rnd_hid_do_all_gui_plugins(NULL, load_all_gui_plugins_cb);
+}
+
+static int list_gui_cb(void *ctx, const char *name, const char *dir, const char *fn)
+{
+	gds_t *tmp = ctx;
+	char *end, *line, buff[1024], *desc = "<description not available>";
+	FILE *f;
+
+	tmp->used = 0;
+	gds_append_str(tmp, dir);
+	gds_append(tmp, RND_DIR_SEPARATOR_C);
+	gds_append_str(tmp, fn);
+
+	f = rnd_fopen(NULL, tmp->array, "r");
+	if (f != NULL) {
+		while((line = fgets(buff, sizeof(buff), f)) != NULL) {
+			while(isspace(*line)) line++;
+			if (strncmp(line, "$short", 6) == 0) {
+				desc = line + 6;
+				while(isspace(*desc)) desc++;
+				end = strpbrk(desc, "\r\n");
+				if (end != NULL)
+					*end = '\0';
+				break;
+			}
+		}
+		fclose(f);
+	}
+
+	fprintf(stderr, "\t%-16s %s\n", name, desc);
+	return 0;
+}
+
+void rnd_hid_print_all_gui_plugins(void)
+{
+	gds_t tmp = {0};
+	rnd_hid_do_all_gui_plugins(&tmp, list_gui_cb);
+	gds_uninit(&tmp);
+}
+
+
 /* low level (pup) load a plugin if it is not already loaded; if plugname is
    NULL, try to load all plugins */
 static void rnd_hid_load_gui_plugin(const char *plugname)
@@ -300,6 +354,9 @@ rnd_hid_t *rnd_hid_find_gui(rnd_hidlib_t *hidlib, const char *preference)
 	rnd_hid_t *gui;
 	int i;
 
+	if (strncmp(preference, "hid_", 4) == 0)
+		preference += 4;
+
 	/* ugly hack for historical reasons: some old configs and veteran users are used to the --gui gtk option */
 	if ((preference != NULL) && (strcmp(preference, "gtk") == 0)) {
 		gui = rnd_hid_find_gui(hidlib, "gtk2_gl");
@@ -310,6 +367,10 @@ rnd_hid_t *rnd_hid_find_gui(rnd_hidlib_t *hidlib, const char *preference)
 		if (gui != NULL)
 			return gui;
 
+		gui = rnd_hid_find_gui(hidlib, "gtk4_gl");
+		if (gui != NULL)
+			return gui;
+
 		return NULL;
 	}
 
@@ -436,6 +497,99 @@ void rnd_pcbhl_conf_postproc(void)
 	rnd_conf_force_set_str(rnd_conf.rc.path.home, get_homedir()); rnd_conf_ro("rc/path/home");
 }
 
+char *rnd_exec_prefix(char *argv0, const char *bin_dir, const char *bin_dir_to_execprefix)
+{
+	size_t l;
+	int haspath;
+	char *t1, *t2;
+	int found_bindir = 0;
+	char *exec_prefix = NULL;
+	char *bindir = NULL;
+
+
+	/* see if argv0 has enough of a path to let lrealpath give the
+	   real path.  This should be the case if you invoke pcb with
+	   something like /usr/local/bin/pcb or ./pcb or ./foo/pcb
+	   but if you just use pcb and it exists in your path, you'll
+	   just get back pcb again. */
+	haspath = (strchr(argv0, RND_DIR_SEPARATOR_C) != NULL);
+
+#ifdef DEBUG
+	printf("rnd_exec_prefix (%s): haspath = %d\n", argv0, haspath);
+#endif
+
+	if (haspath) {
+		bindir = rnd_lrealpath(argv0);
+		if (bindir == NULL)
+			bindir = rnd_strdup(argv0);
+		found_bindir = 1;
+	}
+	else {
+		char *path, *p, *tmps;
+		struct stat sb;
+		int r;
+
+		tmps = getenv("PATH");
+
+		if (tmps != NULL) {
+			path = rnd_strdup(tmps);
+
+			/* search through the font path for a font file */
+			for (p = strtok(path, RND_PATH_DELIMETER); p && *p; p = strtok(NULL, RND_PATH_DELIMETER)) {
+#ifdef DEBUG
+				printf("Looking for %s in %s\n", argv0, p);
+#endif
+				if ((tmps = (char *) malloc((strlen(argv0) + strlen(p) + 2) * sizeof(char))) == NULL) {
+					fprintf(stderr, "rnd_exec_prefix():  malloc failed\n");
+					exit(1);
+				}
+				sprintf(tmps, "%s%s%s", p, RND_DIR_SEPARATOR_S, argv0);
+				r = stat(tmps, &sb);
+				if (r == 0) {
+#ifdef DEBUG
+					printf("Found it:  \"%s\"\n", tmps);
+#endif
+					bindir = rnd_lrealpath(tmps);
+					if (bindir == NULL)
+						bindir = rnd_strdup(tmps);
+					found_bindir = 1;
+					free(tmps);
+					break;
+				}
+				free(tmps);
+			}
+			free(path);
+		}
+	}
+
+	if (found_bindir) {
+		/* strip off the executable name leaving only the path */
+		t2 = NULL;
+		t1 = strchr(bindir, RND_DIR_SEPARATOR_C);
+		while (t1 != NULL && *t1 != '\0') {
+			t2 = t1;
+			t1 = strchr(t2 + 1, RND_DIR_SEPARATOR_C);
+		}
+		if (t2 != NULL)
+			*t2 = '\0';
+	}
+	else {
+		/* we have failed to find out anything from argv[0] so fall back to the original install prefix */
+		bindir = rnd_strdup(bin_dir);
+	}
+
+	/* now find the path to exec_prefix */
+	l = strlen(bindir) + 1 + strlen(bin_dir_to_execprefix) + 1;
+	if ((exec_prefix = (char *) malloc(l * sizeof(char))) == NULL) {
+		fprintf(stderr, "rnd_exec_prefix():  malloc failed\n");
+		exit(1);
+	}
+	sprintf(exec_prefix, "%s%s%s", bindir, RND_DIR_SEPARATOR_S, bin_dir_to_execprefix);
+	free(bindir);
+	return exec_prefix;
+}
+
+
 extern void rnd_menu_init1(void);
 
 void rnd_hidlib_init1(void (*conf_core_init)(void))
@@ -667,7 +821,7 @@ void rnd_fix_locale_and_env_()
 		wchar_t *w, wexedir[MAX_PATH];
 
 		if (!GetModuleFileNameW(NULL, wexedir, MAX_PATH)) {
-			fprintf(stderr, "pcb-rnd: GetModuleFileNameW(): failed to determine executable full path\n");
+			fprintf(stderr, "%s: GetModuleFileNameW(): failed to determine executable full path\n", rnd_app.package);
 			exit(1);
 		}
 
@@ -786,18 +940,31 @@ int rnd_main_args_add(rnd_main_args_t *ga, char *cmd, char *arg)
 	return 0;
 }
 
-int rnd_main_args_setup1(rnd_main_args_t *ga)
+static int apply_plugin_cli_conf(rnd_main_args_t *ga, int relax)
 {
 	int n;
-	/* Now that plugins are already initialized, apply plugin config items */
 	for(n = 0; n < vtp0_len(&ga->plugin_cli_conf); n++) {
 		const char *why, *arg = ga->plugin_cli_conf.array[n];
+
+		if (ga->plugin_cli_conf.array[n] == NULL) continue;
+
 		if (rnd_conf_set_from_cli(NULL, arg, NULL, &why) != 0) {
-			fprintf(stderr, "Error: failed to set config %s: %s\n", arg, why);
-			return 1;
+			if (!relax) {
+				fprintf(stderr, "Error: failed to set config %s: %s\n", arg, why);
+				return 1;
+			}
 		}
+		else
+			ga->plugin_cli_conf.array[n] = NULL; /* remove if succesfully set so it won't be set again on multiple calls at different stages */
 	}
-	vtp0_uninit(&ga->plugin_cli_conf);
+	return 0;
+}
+
+int rnd_main_args_setup1(rnd_main_args_t *ga)
+{
+	/* Now that plugins are already initialized, apply plugin config items;
+	   non-existing ones are not fatal as the GUI plugin is not yet loaded */
+	apply_plugin_cli_conf(ga, 1);
 
 	/* Export pcb from command line if requested.  */
 	switch(ga->do_what) {
@@ -833,6 +1000,13 @@ int rnd_main_args_setup1(rnd_main_args_t *ga)
 	if (!rnd_gui)
 		return 1;
 
+	/* Now that even the GUI plugin is initialized, apply remaining plugin config
+	   items; non-existing ones are fatal */
+	if (apply_plugin_cli_conf(ga, 0) != 0)
+		return 1;
+
+	vtp0_uninit(&ga->plugin_cli_conf);
+
 	return 0;
 }
 
