@@ -51,6 +51,7 @@
 typedef struct {
 	void *caller_data; /* WARNING: for now, this must be the first field (see core spinbox enter_cb) */
 	rnd_gtk_t *gctx;
+	rnd_hidlib_t *hidlib; /* active hidlib at the moment the dialog box was created */
 	rnd_hid_attribute_t *attrs;
 	GtkWidget **wl;     /* content widget */
 	GtkWidget **wltop;  /* the parent widget, which is different from wl if reparenting (extra boxes, e.g. for framing or scrolling) was needed */
@@ -66,6 +67,8 @@ typedef struct {
 	unsigned freeing_gui:1;
 	unsigned being_destroyed:1;
 	unsigned modal:1;
+	unsigned placed:1;
+	unsigned mapped:1;
 } attr_dlg_t;
 
 #define change_cb(ctx, dst) \
@@ -216,7 +219,7 @@ typedef struct {
    directly in the scrolled window without adding an extra box. Useful
    when scrolled content wants to interact with the scroll, e.g. tree table
    wants to scroll where the selection is */
-static GtkWidget *frame_scroll_(GtkWidget *parent, rnd_hatt_compflags_t flags, GtkWidget **wltop, GtkWidget *inner)
+static GtkWidget *frame_scroll_(GtkWidget *parent, rnd_hatt_compflags_t flags, GtkWidget **wltop, GtkWidget *inner, int horiz)
 {
 	GtkWidget *fr;
 	int expfill = (flags & RND_HATF_EXPFILL);
@@ -226,7 +229,10 @@ static GtkWidget *frame_scroll_(GtkWidget *parent, rnd_hatt_compflags_t flags, G
 		fr = gtk_frame_new(NULL);
 		gtkc_box_pack_append(parent, fr, expfill, 0);
 
-		parent = gtkc_hbox_new(FALSE, 0);
+		if (horiz)
+			parent = gtkc_hbox_new(FALSE, 0);
+		else
+			parent = gtkc_vbox_new(FALSE, 0);
 		gtkc_frame_set_child(fr, parent);
 		if (wltop != NULL) {
 			*wltop = fr;
@@ -263,7 +269,12 @@ static GtkWidget *frame_scroll_(GtkWidget *parent, rnd_hatt_compflags_t flags, G
 
 static GtkWidget *frame_scroll(GtkWidget *parent, rnd_hatt_compflags_t flags, GtkWidget **wltop)
 {
-	return frame_scroll_(parent, flags, wltop, NULL);
+	return frame_scroll_(parent, flags, wltop, NULL, 1);
+}
+
+static GtkWidget *frame_scrollhv(GtkWidget *parent, rnd_hatt_compflags_t flags, GtkWidget **wltop, int horiz)
+{
+	return frame_scroll_(parent, flags, wltop, NULL, horiz);
 }
 
 
@@ -289,10 +300,7 @@ typedef struct {
 
 static int rnd_gtk_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, rnd_gtk_attr_tb_t *tb_st, int start_from);
 
-#include "dlg_attr_tree.c"
-#include "dlg_attr_misc.c"
-#include "dlg_attr_txt.c"
-#include "dlg_attr_box.c"
+#include DLG_INCLUDES
 
 static int rnd_gtk_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, rnd_gtk_attr_tb_t *tb_st, int start_from)
 {
@@ -339,7 +347,7 @@ static int rnd_gtk_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, rnd_gtk
 		/* create the actual widget from attrs */
 		switch (ctx->attrs[j].type) {
 			case RND_HATT_BEGIN_HBOX:
-				bparent = frame_scroll(parent, ctx->attrs[j].rnd_hatt_flags, &ctx->wltop[j]);
+				bparent = frame_scrollhv(parent, ctx->attrs[j].rnd_hatt_flags, &ctx->wltop[j], 0);
 				hbox = gtkc_hbox_new(FALSE, ((ctx->attrs[j].rnd_hatt_flags & RND_HATF_TIGHT) ? 0 : 4));
 				gtkc_box_pack_append(bparent, hbox, expfill, 0);
 				ctx->wl[j] = hbox;
@@ -347,7 +355,7 @@ static int rnd_gtk_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, rnd_gtk
 				break;
 
 			case RND_HATT_BEGIN_VBOX:
-				bparent = frame_scroll(parent, ctx->attrs[j].rnd_hatt_flags, &ctx->wltop[j]);
+				bparent = frame_scrollhv(parent, ctx->attrs[j].rnd_hatt_flags, &ctx->wltop[j], 1);
 				vbox1 = gtkc_vbox_new(FALSE, ((ctx->attrs[j].rnd_hatt_flags & RND_HATF_TIGHT) ? 0 : 4));
 				expfill = (ctx->attrs[j].rnd_hatt_flags & RND_HATF_EXPFILL);
 				gtkc_box_pack_append(bparent, vbox1, expfill, 0);
@@ -697,6 +705,8 @@ static int rnd_gtk_attr_dlg_set(attr_dlg_t *ctx, int idx, const rnd_hid_attr_val
 static gint rnd_gtk_attr_dlg_configure_event_cb(GtkWidget *widget, long x, long y, long z, gpointer data)
 {
 	attr_dlg_t *ctx = (attr_dlg_t *)data;
+	if (!ctx->placed)
+		return FALSE;
 	return rnd_gtk_winplace_cfg(ctx->gctx->hidlib, widget, ctx, ctx->id);
 }
 
@@ -716,6 +726,8 @@ static void rnd_gtk_attr_dlg_free_gui(attr_dlg_t *ctx)
 
 	for(i = 0; i < ctx->n_attrs; i++) {
 		switch(ctx->attrs[i].type) {
+			case RND_HATT_BEGIN_HPANE: /* fall through */
+			case RND_HATT_BEGIN_VPANE: rnd_gtk_pane_pre_free(ctx, &ctx->attrs[i], i); break;
 			case RND_HATT_TREE: rnd_gtk_tree_pre_free(ctx, &ctx->attrs[i], i); break;
 			case RND_HATT_BUTTON: g_signal_handlers_block_by_func(G_OBJECT(ctx->wl[i]), G_CALLBACK(button_changed_cb), &(ctx->attrs[i])); break;
 			case RND_HATT_PREVIEW: rnd_gtk_preview_del(ctx->gctx, RND_GTK_PREVIEW(ctx->wl[i]));
@@ -808,7 +820,45 @@ static void rnd_gtk_initial_wstates(attr_dlg_t *ctx)
 			rnd_gtk_attr_dlg_widget_hide_(ctx, n, 1);
 }
 
-void *rnd_gtk_attr_dlg_new(rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny)
+RND_INLINE void rnd_gtk_attr_dlg_place(attr_dlg_t *ctx, int *plc, int defx, int defy)
+{
+	if (rnd_conf.editor.auto_place) {
+		if ((plc[2] > 0) && (plc[3] > 0))
+			gtkc_window_resize(GTK_WINDOW(ctx->dialog), plc[2], plc[3]);
+		if ((plc[0] >= 0) && (plc[1] >= 0))
+			gtkc_window_move(GTK_WINDOW(ctx->dialog), plc[0], plc[1]);
+	}
+	else if ((defx > 0) && (defy > 0))
+		gtkc_window_resize(GTK_WINDOW(ctx->dialog), defx, defy);
+}
+
+typedef struct {
+	attr_dlg_t *ctx;
+	int plc[4];
+	int defx, defy;
+} attr_dlg_timed_place_t;
+
+
+static gboolean attr_dlg_timed_place(void *udata)
+{
+	attr_dlg_timed_place_t *tp = udata;
+
+	if (!tp->ctx->mapped)
+		return TRUE; /* try again later */
+
+	rnd_gtk_attr_dlg_place(tp->ctx, tp->plc, tp->defx, tp->defy);
+	tp->ctx->placed = 1;
+	free(tp);
+	return FALSE; /* remove */
+}
+
+void rnd_gtk_attr_dlg_mapped_cb(GObject *self, gpointer user_data)
+{
+	attr_dlg_t *ctx = user_data;
+	ctx->mapped = 1;
+}
+
+void *rnd_gtk_attr_dlg_new(rnd_hid_t *hid, rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny)
 {
 	GtkWidget *main_vbox;
 	attr_dlg_t *ctx;
@@ -821,6 +871,7 @@ void *rnd_gtk_attr_dlg_new(rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t
 	ctx = calloc(sizeof(attr_dlg_t), 1);
 
 	ctx->gctx = gctx;
+	ctx->hidlib = gctx->hidlib;
 	ctx->attrs = attrs;
 	ctx->n_attrs = n_attrs;
 	ctx->wl = calloc(sizeof(GtkWidget *), n_attrs);
@@ -837,18 +888,29 @@ void *rnd_gtk_attr_dlg_new(rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t
 	if ((modal && rnd_gtk_conf_hid.plugins.hid_gtk.dialog.transient_modal) || (!modal && rnd_gtk_conf_hid.plugins.hid_gtk.dialog.transient_modeless))
 		gtk_window_set_transient_for(GTK_WINDOW(ctx->dialog), GTK_WINDOW(gctx->wtop_window));
 
+	g_signal_connect(ctx->dialog, "map", G_CALLBACK(rnd_gtk_attr_dlg_mapped_cb), ctx);
+
+	if (!GTKC_TIMED_WINDOW_PLACEMENT) {
+		/* do the placement immediately, gtk won't interfere (gtk2) */
+		rnd_gtk_attr_dlg_place(ctx, plc, defx, defy);
+		ctx->placed = 1;
+	}
+	else {
+		/* cheat: do the placement after a while because gtk interferes (gtk4) */
+		attr_dlg_timed_place_t *tp = malloc(sizeof(attr_dlg_timed_place_t));
+		tp->ctx = ctx;
+		memcpy(tp->plc, plc, sizeof(plc));
+		tp->defx = defx;
+		tp->defy = defy;
+		g_timeout_add(20, attr_dlg_timed_place, tp);
+		ctx->placed = 0;
+	}
+
+
 	gtk_window_set_title(GTK_WINDOW(ctx->dialog), title);
 	gtkc_window_set_role(GTK_WINDOW(ctx->dialog), id); /* optional hint for the window manager for figuring session placement/restore */
 	gtk_window_set_modal(GTK_WINDOW(ctx->dialog), modal);
 
-	if (rnd_conf.editor.auto_place) {
-		if ((plc[2] > 0) && (plc[3] > 0))
-			gtkc_window_resize(GTK_WINDOW(ctx->dialog), plc[2], plc[3]);
-		if ((plc[0] >= 0) && (plc[1] >= 0))
-			gtkc_window_move(GTK_WINDOW(ctx->dialog), plc[0], plc[1]);
-	}
-	else if ((defx > 0) && (defy > 0))
-		gtkc_window_resize(GTK_WINDOW(ctx->dialog), defx, defy);
 
 	gtk2c_bind_win_resize(ctx->dialog, rnd_gtkc_xy_ev(&ctx->ev_resize, rnd_gtk_attr_dlg_configure_event_cb, ctx));
 	ctx->destroy_handler = gtkc_bind_win_destroy(ctx->dialog, rnd_gtkc_xy_ev(&ctx->ev_destroy, rnd_gtk_attr_dlg_destroy_event_cb, ctx));
@@ -859,6 +921,7 @@ void *rnd_gtk_attr_dlg_new(rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t
 	rnd_gtk_attr_dlg_add(ctx, main_vbox, NULL, 0);
 
 	gtkc_widget_show_all(ctx->dialog);
+	gtk_widget_realize(ctx->dialog);
 	gtk4c_bind_win_resize(ctx->dialog, rnd_gtkc_xy_ev(&ctx->ev_resize, rnd_gtk_attr_dlg_configure_event_cb, ctx));
 
 	rnd_gtk_initial_wstates(ctx);
@@ -866,6 +929,7 @@ void *rnd_gtk_attr_dlg_new(rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t
 	if (rnd_gtk_conf_hid.plugins.hid_gtk.dialog.auto_present)
 		gtk_window_present(GTK_WINDOW(ctx->dialog));
 
+
 	return ctx;
 }
 
@@ -876,6 +940,7 @@ void *rnd_gtk_attr_sub_new(rnd_gtk_t *gctx, GtkWidget *parent_box, rnd_hid_attri
 	ctx = calloc(sizeof(attr_dlg_t), 1);
 
 	ctx->gctx = gctx;
+	ctx->hidlib = gctx->hidlib;
 	ctx->attrs = attrs;
 	ctx->n_attrs = n_attrs;
 	ctx->wl = calloc(sizeof(GtkWidget *), n_attrs);
@@ -1058,10 +1123,8 @@ int rnd_gtk_winplace_cfg(rnd_hidlib_t *hidlib, GtkWidget *widget, void *ctx, con
 
 	gtkc_widget_get_allocation(widget, &allocation);
 
-#ifdef RND_FOR_GTK2
-	/* For whatever reason, get_allocation doesn't set these. Gtk. */
-	gtk_window_get_position(GTK_WINDOW(widget), &allocation.x, &allocation.y);
-#endif
+	/* allocation x and y are window-relative, need to get window origin searately */
+	gtkc_window_get_position(GTK_WINDOW(widget), &allocation.x, &allocation.y);
 
 	rnd_event(hidlib, RND_EVENT_DAD_NEW_GEO, "psiiii", ctx, id,
 		(int)allocation.x, (int)allocation.y, (int)allocation.width, (int)allocation.height);
@@ -1072,5 +1135,8 @@ int rnd_gtk_winplace_cfg(rnd_hidlib_t *hidlib, GtkWidget *widget, void *ctx, con
 rnd_hidlib_t *rnd_gtk_attr_get_dad_hidlib(void *hid_ctx)
 {
 	attr_dlg_t *ctx = hid_ctx;
+
+	if (rnd_hid_enable_per_dialog_hidlib)
+		return ctx->hidlib;
 	return ctx->gctx->hidlib;
 }
