@@ -201,6 +201,10 @@ static void v_adjustment_changed_cb(GtkAdjustment *adj, rnd_gtk_topwin_t *tw)
 /* Save size of top window changes so PCB can restart at its size at exit. */
 static gint top_window_configure_event_cb(GtkWidget *widget, long x, long y, long z, void *tw_)
 {
+	rnd_gtk_topwin_t *tw = tw_;
+	if (!tw->placed)
+		return FALSE;
+
 	return rnd_gtk_winplace_cfg(ghidgui->hidlib, widget, NULL, "top");
 }
 
@@ -267,11 +271,11 @@ static const char * FullScreen_xpm[] = {
 " .      ",
 "        "};
 
-/* Embed an XPM image in a button, and make it display as small as possible 
- *   Returns: a new image button. When freeing the object, the image needs to be freed 
- *            as well, using :
- *            g_object_unref(gtk_button_get_image(GTK_BUTTON(button)); g_object_unref(button);
- */
+/* Embed an XPM image in a button, and make it display as small as possible
+   Returns: a new image button. When freeing the object, the image needs to
+   be freed as well, using:
+   g_object_unref(gtk_button_get_image(GTK_BUTTON(button));
+   g_object_unref(button); */
 static GtkWidget *create_image_button_from_xpm_data(const char **xpm_data)
 {
 	GtkWidget *button;
@@ -359,14 +363,14 @@ static void rnd_gtk_build_top_window(rnd_gtk_t *ctx, rnd_gtk_topwin_t *tw)
 	/* This box will be made insensitive when the gui needs
 	   a modal button GetLocation button press. */
 	tw->left_toolbar = gtkc_vbox_new(FALSE, 0);
-	gtkc_paned_pack1(GTK_PANED(tw->hpaned_middle), tw->left_toolbar, FALSE);
+	gtkc_paned_pack1(tw->hpaned_middle, tw->left_toolbar, FALSE);
 
 	tw->dockbox[RND_HID_DOCK_LEFT] = gtkc_vbox_new(FALSE, 8);
 	gtkc_box_pack_append(tw->left_toolbar, tw->dockbox[RND_HID_DOCK_LEFT], TRUE, 0);
 
 	/* -- main content */
 	tw->vbox_middle = gtkc_vbox_new(FALSE, 0);
-	gtkc_paned_pack2(GTK_PANED(tw->hpaned_middle), tw->vbox_middle, TRUE);
+	gtkc_paned_pack2(tw->hpaned_middle, tw->vbox_middle, TRUE);
 
 
 	/* -- The PCB layout output drawing area */
@@ -428,10 +432,6 @@ static void rnd_gtk_build_top_window(rnd_gtk_t *ctx, rnd_gtk_topwin_t *tw)
 	rnd_gtk_command_combo_box_entry_create(&tw->cmd, rnd_gtk_topwin_hide_status, tw);
 	gtkc_box_pack_append(tw->bottom_hbox, tw->cmd.command_combo_box, FALSE, 0);
 
-
-	/* optional resize grip: rightmost widget in the status line hbox */
-	gtkc_create_resize_grip(tw->bottom_hbox);
-
 	gtkc_bind_mouse_enter(tw->drawing_area, rnd_gtkc_xy_ev(&ghidgui->wtop_enter, drawing_area_enter_cb, tw));
 	gtk2c_bind_win_resize(ghidgui->wtop_window, rnd_gtkc_xy_ev(&ghidgui->wtop_rs, top_window_configure_event_cb, tw));
 	gtkc_bind_win_delete(ghidgui->wtop_window, rnd_gtkc_xy_ev(&ghidgui->wtop_del, delete_chart_cb, ctx));
