@@ -182,7 +182,7 @@ static void rnd_gtkg_do_exit_(rnd_gtk_t *gctx)
 static void rnd_gtkg_do_exit(rnd_hid_t *hid);
 
 
-static void rnd_gtk_topwinplace(rnd_hidlib_t *hidlib, GtkWidget *dialog, const char *id)
+void rnd_gtk_topwinplace(rnd_hidlib_t *hidlib, GtkWidget *dialog, const char *id)
 {
 	int plc[4] = {-1, -1, -1, -1};
 
@@ -196,6 +196,14 @@ static void rnd_gtk_topwinplace(rnd_hidlib_t *hidlib, GtkWidget *dialog, const c
 	}
 }
 
+static gboolean topwin_timed_place(void *udata)
+{
+	rnd_gtk_t *gctx = udata;
+	rnd_gtk_topwinplace(gctx->hidlib, gctx->wtop_window, "top");
+	gctx->topwin.placed = 1;
+	return FALSE; /* remove */
+}
+
 static void rnd_gtk_topwin_create(rnd_gtk_t *gctx, int *argc, char ***argv)
 {
 	GtkWidget *window;
@@ -207,9 +215,19 @@ static void rnd_gtk_topwin_create(rnd_gtk_t *gctx, int *argc, char ***argv)
 	gctx->impl.init_renderer(argc, argv, &gctx->port);
 	gctx->wtop_window = window = gctx->port.top_window = gtkc_topwin_new();
 
-	rnd_gtk_topwinplace(gctx->hidlib, window, "top");
-	gtk_window_set_title(GTK_WINDOW(window), rnd_app.package);
+	if (!GTKC_TIMED_WINDOW_PLACEMENT) {
+		/* do the placement immediately, gtk won't interfere (gtk2) */
+		rnd_gtk_topwinplace(gctx->hidlib, window, "top");
+		gctx->topwin.placed = 1;
+	}
+	else {
+		/* cheat: do the placement after a while because gtk interferes (gtk4) */
+		g_timeout_add(500, topwin_timed_place, gctx);
+		gctx->topwin.placed = 0;
+	}
 
+
+	gtk_window_set_title(GTK_WINDOW(window), rnd_app.package);
 	gtkc_widget_show_all(gctx->port.top_window);
 }
 
@@ -264,7 +282,7 @@ static char *rnd_gtkg_fileselect(rnd_hid_t *hid, const char *title, const char *
 
 static void *rnd_gtk_attr_dlg_new_(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny)
 {
-	return rnd_gtk_attr_dlg_new((rnd_gtk_t *)hid->hid_data, id, attrs, n_attrs, title, caller_data, modal, button_cb, defx, defy, minx, miny);
+	return rnd_gtk_attr_dlg_new(hid, (rnd_gtk_t *)hid->hid_data, id, attrs, n_attrs, title, caller_data, modal, button_cb, defx, defy, minx, miny);
 }
 
 static void rnd_gtkg_beep(rnd_hid_t *hid);
@@ -461,6 +479,16 @@ static void rnd_gtkg_set_hidlib(rnd_hid_t *hid, rnd_hidlib_t *hidlib)
 	rnd_gtk_zoom_view_win(&gctx->port.view, 0, 0, hidlib->size_x, hidlib->size_y, 0);
 }
 
+static rnd_hidlib_t *rnd_gtkg_get_hidlib(rnd_hid_t *hid)
+{
+	rnd_gtk_t *gctx = hid->hid_data;
+
+	if (gctx == NULL)
+		return NULL;
+
+	return gctx->hidlib;
+}
+
 static void rnd_gtkg_reg_mouse_cursor(rnd_hid_t *hid, int idx, const char *name, const unsigned char *pixel, const unsigned char *mask)
 {
 	rnd_gtk_reg_mouse_cursor((rnd_gtk_t *)hid->hid_data, idx, name, pixel, mask);
@@ -551,6 +579,9 @@ static void rnd_gtkg_init_pixmap(rnd_hid_t *hid, rnd_pixmap_t *pxm)
 
 static void rnd_gtkg_uninit_pixmap_(rnd_hid_t *hid, rnd_pixmap_t *pxm)
 {
+	rnd_gtk_t *gctx = hid->hid_data;
+	if ((gctx != NULL) && (gctx->impl.uninit_pixmap != NULL))
+		gctx->impl.uninit_pixmap(gctx->hidlib, (rnd_gtk_pixmap_t *)(pxm->hid_data));
 	rnd_gtkg_uninit_pixmap_low((rnd_gtk_pixmap_t *)(pxm->hid_data));
 	free(pxm->hid_data);
 }
@@ -650,6 +681,7 @@ void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
 	dst->busy = rnd_gtkg_busy;
 
 	dst->set_hidlib = rnd_gtkg_set_hidlib;
+	dst->get_hidlib = rnd_gtkg_get_hidlib;
 	dst->get_dad_hidlib = rnd_gtk_attr_get_dad_hidlib;
 
 	dst->key_state = &rnd_gtk_keymap;
