@@ -16,10 +16,7 @@
 #include <librnd/plugins/lib_gtk_common/coord_conv.h>
 #include <librnd/plugins/lib_gtk_common/glue_common.h>
 
-#include <librnd/plugins/lib_hid_gl/opengl.h>
-#include <librnd/plugins/lib_hid_gl/draw_gl.h>
 #include <librnd/plugins/lib_hid_gl/hidgl.h>
-#include <librnd/plugins/lib_hid_gl/stencil_gl.h>
 
 #include <librnd/plugins/lib_gtk_common/hid_gtk_conf.h>
 
@@ -71,20 +68,19 @@ int ghid_gl_set_layer_group(rnd_hid_t *hid, rnd_layergrp_id_t group, const char
 {
 	rnd_hidlib_t *hidlib = ghidgui->hidlib;
 	render_priv_t *priv = ghidgui->port.render_priv;
+	double tx, ty, zx, zy, zz;
 
-	glMatrixMode(GL_MODELVIEW);
-	glLoadIdentity();
-	glTranslatef(0.0f, 0.0f, -HIDGL_Z_NEAR);
-
-	glScalef((rnd_conf.editor.view.flip_x ? -1. : 1.) / ghidgui->port.view.coord_per_px, (rnd_conf.editor.view.flip_y ? -1. : 1.) / ghidgui->port.view.coord_per_px, ((rnd_conf.editor.view.flip_x == rnd_conf.editor.view.flip_y) ? 1. : -1.) / ghidgui->port.view.coord_per_px);
-	glTranslatef(rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->size_x : -ghidgui->port.view.x0, rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->size_y : -ghidgui->port.view.y0, 0);
+	tx = rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->size_x : -ghidgui->port.view.x0;
+	ty = rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->size_y : -ghidgui->port.view.y0;
+	zx = (rnd_conf.editor.view.flip_x ? -1. : 1.) / ghidgui->port.view.coord_per_px;
+	zy = (rnd_conf.editor.view.flip_y ? -1. : 1.) / ghidgui->port.view.coord_per_px;
+	zz = ((rnd_conf.editor.view.flip_x == rnd_conf.editor.view.flip_y) ? 1. : -1.) / ghidgui->port.view.coord_per_px;
+	hidgl_set_view(tx, ty, zx, zy, zz);
 
 	/* Put the renderer into a good state so that any drawing is done in standard mode */
 
-	drawgl_flush();
-	drawgl_reset();
-	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-	glDisable(GL_STENCIL_TEST);
+	hidgl_flush_drawing();
+	hidgl_reset();
 
 	priv->trans_lines = rnd_true;
 	return 1;
@@ -92,8 +88,8 @@ int ghid_gl_set_layer_group(rnd_hid_t *hid, rnd_layergrp_id_t group, const char
 
 static void ghid_gl_end_layer(rnd_hid_t *hid)
 {
-	drawgl_flush();
-	drawgl_reset();
+	hidgl_flush_drawing();
+	hidgl_reset();
 }
 
 void ghid_gl_destroy_gc(rnd_hid_gc_t gc)
@@ -126,21 +122,23 @@ void ghid_gl_draw_grid_local(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_coord_t c
 static void ghid_gl_draw_grid(rnd_hidlib_t *hidlib, rnd_box_t *drawn_area)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
+	rnd_coord_t grd = hidlib->grid;
 
-	if ((Vz(hidlib->grid) < RND_MIN_GRID_DISTANCE) || (!rnd_conf.editor.draw_grid))
-		return;
+	if (grd <= 0)
+		grd = 1;
 
-	glEnable(GL_COLOR_LOGIC_OP);
-	glLogicOp(GL_XOR);
+	if ((Vz(hidlib->grid) < RND_MIN_GRID_DISTANCE) || (!rnd_conf.editor.draw_grid)) {
+		if (!rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.sparse)
+			return;
+		grd *= (rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.min_dist_px / Vz(grd));
+	}
 
-	glColor3f(priv->grid_color.fr, priv->grid_color.fg, priv->grid_color.fb);
+	hidgl_set_grid_color(priv->grid_color.fr, priv->grid_color.fg, priv->grid_color.fb);
 
 	if (rnd_gtk_conf_hid.plugins.hid_gtk.local_grid.enable)
-		hidgl_draw_local_grid(hidlib, grid_local_x, grid_local_y, grid_local_radius, ghidgui->port.view.coord_per_px, rnd_conf.editor.cross_grid);
+		hidgl_draw_local_grid(hidlib, grd, grid_local_x, grid_local_y, grid_local_radius, ghidgui->port.view.coord_per_px, rnd_conf.editor.cross_grid);
 	else
-		hidgl_draw_grid(hidlib, drawn_area, ghidgui->port.view.coord_per_px, rnd_conf.editor.cross_grid);
-
-	glDisable(GL_COLOR_LOGIC_OP);
+		hidgl_draw_grid(hidlib, grd, drawn_area, ghidgui->port.view.coord_per_px, rnd_conf.editor.cross_grid);
 }
 
 static void rnd_gl_draw_texture(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t bw, rnd_coord_t bh)
@@ -148,10 +146,15 @@ static void rnd_gl_draw_texture(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd
 	hidgl_draw_texture_rect(ox, oy, ox+bw, oy+bh, gpm->cache.lng);
 }
 
+static void ghid_gl_uninit_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm)
+{
+	if (gpm->cache.lng != 0)
+		hidgl_texture_free(gpm->cache.lng);
+}
+
 static void ghid_gl_draw_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t bw, rnd_coord_t bh)
 {
-	GLuint texture_handle = gpm->cache.lng;
-	if (texture_handle == 0) {
+	if (gpm->cache.lng == 0) {
 		int width = gpm->pxm->sx;
 		int height = gpm->pxm->sy;
 		int rowstride = gdk_pixbuf_get_rowstride(gpm->image);
@@ -162,14 +165,11 @@ static void ghid_gl_draw_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd
 		g_warn_if_fail(bits_per_sample == 8);
 		g_warn_if_fail(rowstride == width * n_channels);
 
-		glGenTextures(1, &texture_handle);
-		glBindTexture(GL_TEXTURE_2D, texture_handle);
-
-		/* XXX: We should probably determine what the maximum texture supported is,
-		        and if our image is larger, shrink it down using GDK pixbuf routines
-		        rather than having it fail below. */
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, (n_channels == 4) ? GL_RGBA : GL_RGB, GL_UNSIGNED_BYTE, pixels);
-		gpm->cache.lng = texture_handle;
+		TODO(
+			"We should probably determine what the maximum texture supported is,"
+			"and if our image is larger, shrink it down using GDK pixbuf routines"
+			"rather than having it fail below.");
+		gpm->cache.lng = hidgl_texture_import(pixels, width, height, (n_channels == 4));
 	}
 
 	rnd_gl_draw_texture(hidlib, gpm, ox, oy, bw, bh);
@@ -243,9 +243,14 @@ void ghid_gl_set_line_cap(rnd_hid_gc_t gc, rnd_cap_style_t style)
 	   low level line draw function has to build the ap from triangles */
 }
 
+
+/* gc width in coords */
+#define GCWC(gc) ((gc->width < 0) ?  (-gc->width * ghidgui->port.view.coord_per_px) : gc->width)
+
+
 void ghid_gl_set_line_width(rnd_hid_gc_t gc, rnd_coord_t width)
 {
-	gc->width = width < 0 ? (-width) * ghidgui->port.view.coord_per_px : width;
+	gc->width = width;
 }
 
 
@@ -278,14 +283,14 @@ static void ghid_gl_draw_line(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, r
 {
 	USE_GC(gc);
 
-	hidgl_draw_line(gc->core_gc.cap, gc->width, x1, y1, x2, y2, ghidgui->port.view.coord_per_px);
+	hidgl_draw_line(gc->core_gc.cap, GCWC(gc), x1, y1, x2, y2, ghidgui->port.view.coord_per_px);
 }
 
 static void ghid_gl_draw_arc(rnd_hid_gc_t gc, rnd_coord_t cx, rnd_coord_t cy, rnd_coord_t xradius, rnd_coord_t yradius, rnd_angle_t start_angle, rnd_angle_t delta_angle)
 {
 	USE_GC(gc);
 
-	hidgl_draw_arc(gc->width, cx, cy, xradius, yradius, start_angle, delta_angle, ghidgui->port.view.coord_per_px);
+	hidgl_draw_arc(GCWC(gc), cx, cy, xradius, yradius, start_angle, delta_angle, ghidgui->port.view.coord_per_px);
 }
 
 static void ghid_gl_draw_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2)
@@ -391,8 +396,7 @@ static void ghid_gl_show_crosshair(rnd_hidlib_t *hidlib, gboolean paint_new_loca
 			minx, miny, maxx, maxy);
 	}
 
-	TODO("This shouldn't be here, but it's missing from somewhere else; if removed, moving out from the drawing area and clicking a tool button will recolor the backgrouind on intel");
-	glDisable(GL_COLOR_LOGIC_OP);
+	hidgl_reset();
 }
 
 static void ghid_gl_drawing_area_configure_hook(void *port)
@@ -421,7 +425,7 @@ static void ghid_gl_screen_update(void)
     The color structure holds the wanted solid back-ground color, used to first paint the exposed drawing area. */
 static void rnd_gl_draw_expose_init(int w, int h, int xr, int yr, int wr, int hr, rnd_color_t *bg_c)
 {
-	hidgl_init();
+	hidgl_stencil_init();
 	hidgl_expose_init(w, h, bg_c);
 }
 
@@ -432,6 +436,7 @@ static gboolean ghid_gl_drawing_area_expose_cb_common(rnd_hid_t *hid, GtkWidget
 	render_priv_t *priv = port->render_priv;
 	GtkAllocation allocation;
 	rnd_hid_expose_ctx_t ctx;
+	double tx, ty, zx, zy, zz;
 
 	gtkc_widget_get_allocation(widget, &allocation);
 
@@ -444,20 +449,22 @@ static gboolean ghid_gl_drawing_area_expose_cb_common(rnd_hid_t *hid, GtkWidget
 
 	rnd_gl_draw_expose_init(allocation.width, allocation.height, 0, 0, allocation.width, allocation.height, &priv->offlimits_color);
 
-	glScalef((rnd_conf.editor.view.flip_x ? -1. : 1.) / port->view.coord_per_px, (rnd_conf.editor.view.flip_y ? -1. : 1.) / port->view.coord_per_px, ((rnd_conf.editor.view.flip_x == rnd_conf.editor.view.flip_y) ? 1. : -1.) / port->view.coord_per_px);
-	glTranslatef(rnd_conf.editor.view.flip_x ? port->view.x0 - hidlib->size_x : -port->view.x0, rnd_conf.editor.view.flip_y ? port->view.y0 - hidlib->size_y : -port->view.y0, 0);
+	zx = (rnd_conf.editor.view.flip_x ? -1. : 1.) / port->view.coord_per_px;
+	zy = (rnd_conf.editor.view.flip_y ? -1. : 1.) / port->view.coord_per_px;
+	zz = ((rnd_conf.editor.view.flip_x == rnd_conf.editor.view.flip_y) ? 1. : -1.) / port->view.coord_per_px;
+	tx = rnd_conf.editor.view.flip_x ? port->view.x0 - hidlib->size_x : -port->view.x0;
+	ty = rnd_conf.editor.view.flip_y ? port->view.y0 - hidlib->size_y : -port->view.y0;
+	hidgl_set_view(tx, ty, zx, zy, zz);
 
 	/* Draw PCB background, before PCB primitives */
-	hidgl_draw_solid_rect(0, 0, hidlib->size_x, hidlib->size_y, priv->bg_color.fr, priv->bg_color.fg, priv->bg_color.fb);
+	hidgl_draw_initial_fill(0, 0, hidlib->size_x, hidlib->size_y, priv->bg_color.fr, priv->bg_color.fg, priv->bg_color.fb);
 	ghid_gl_draw_bg_image(hidlib);
 
 	ghid_gl_invalidate_current_gc();
-	glMatrixMode(GL_PROJECTION);
-	glPushMatrix();
+	hidgl_push_matrix(1);
 	rnd_app.expose_main(hid, &ctx, NULL);
-	drawgl_flush();
-	glMatrixMode(GL_PROJECTION);
-	glPopMatrix();
+	hidgl_flush_drawing();
+	hidgl_pop_matrix(1);
 
 	ghid_gl_draw_grid(hidlib, &ctx.view);
 
@@ -467,11 +474,11 @@ static gboolean ghid_gl_drawing_area_expose_cb_common(rnd_hid_t *hid, GtkWidget
 		rnd_app.draw_attached(hidlib, 0);
 	if (rnd_app.draw_marks != NULL)
 		rnd_app.draw_marks(hidlib, 0);
-	drawgl_flush();
+	hidgl_flush_drawing();
 
 	ghid_gl_show_crosshair(hidlib, TRUE, ctx.view.X1, ctx.view.Y1, ctx.view.X2, ctx.view.Y2);
 
-	drawgl_flush();
+	hidgl_flush_drawing();
 
 	ghid_gl_end_drawing(port);
 
@@ -487,6 +494,7 @@ static void ghid_gl_preview_expose_common(rnd_hid_t *hid, rnd_hidlib_t *hidlib,
 	rnd_coord_t save_cpp;
 	rnd_gtk_view_t save_view;
 	int save_width, save_height;
+	double tx, ty, zx, zy, zz;
 
 	/* Setup zoom factor for drawing routines */
 	vw = ctx->view.X2 - ctx->view.X1;
@@ -519,15 +527,20 @@ static void ghid_gl_preview_expose_common(rnd_hid_t *hid, rnd_hidlib_t *hidlib,
 
 	/* call the drawing routine */
 	ghid_gl_invalidate_current_gc();
-	glPushMatrix();
-	glScalef((rnd_conf.editor.view.flip_x ? -1. : 1.) / ghidgui->port.view.coord_per_px, (rnd_conf.editor.view.flip_y ? -1. : 1.) / ghidgui->port.view.coord_per_px, 1);
-	glTranslatef(rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->size_x : -ghidgui->port.view.x0, rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->size_y : -ghidgui->port.view.y0, 0);
+	hidgl_push_matrix(0);
+
+	zx = (rnd_conf.editor.view.flip_x ? -1. : 1.) / ghidgui->port.view.coord_per_px;
+	zy = (rnd_conf.editor.view.flip_y ? -1. : 1.) / ghidgui->port.view.coord_per_px;
+	zz = 1;
+	tx = rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->size_x : -ghidgui->port.view.x0;
+	ty = rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->size_y : -ghidgui->port.view.y0;
+	hidgl_set_view(tx, ty, zx, zy, zz);
 
 	rnd_gui->coord_per_pix = ghidgui->port.view.coord_per_px;
 	expcall(hid, ctx);
 
-	drawgl_flush();
-	glPopMatrix();
+	hidgl_flush_drawing();
+	hidgl_pop_matrix(0);
 
 	ghidgui->port.render_priv->in_context = rnd_false;
 
@@ -542,7 +555,7 @@ static void ghid_gl_preview_expose_common(rnd_hid_t *hid, rnd_hidlib_t *hidlib,
 	ghidgui->port.view.canvas_height = save_height;
 }
 
-void ghid_gl_install_common(rnd_gtk_impl_t *impl, rnd_hid_t *hid)
+int ghid_gl_install_common(rnd_gtk_impl_t *impl, rnd_hid_t *hid)
 {
 	if (impl != NULL) {
 		impl->set_special_colors = ghid_gl_set_special_colors;
@@ -551,6 +564,7 @@ void ghid_gl_install_common(rnd_gtk_impl_t *impl, rnd_hid_t *hid)
 		impl->drawing_area_configure_hook = ghid_gl_drawing_area_configure_hook;
 		impl->map_color = map_color;
 		impl->draw_pixmap = ghid_gl_draw_pixmap;
+		impl->uninit_pixmap = ghid_gl_uninit_pixmap;
 	}
 
 	if (hid != NULL) {
@@ -576,4 +590,6 @@ void ghid_gl_install_common(rnd_gtk_impl_t *impl, rnd_hid_t *hid)
 		hid->set_drawing_mode = hidgl_set_drawing_mode;
 		hid->render_burst = ghid_gl_render_burst;
 	}
+
+	return 0;
 }
