@@ -4,6 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2009-2011 PCB Contributers (See ChangeLog for details)
+ *  Copyright (C) 2021,2022 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,105 +37,181 @@
 
 #include <librnd/core/grid.h>
 #include <librnd/core/hid.h>
-#include "hidgl.h"
 #include <librnd/poly/rtree.h>
 #include <librnd/core/hidlib.h>
 
-#include "draw_gl.c"
-
+#include "draw.h"
+#include "stenc.h"
 #include "stencil_gl.h"
+#include "opengl.h"
 
-void hidgl_init(void)
+#include "hidgl.h"
+
+#include "lib_hid_gl_conf.h"
+extern conf_lib_hid_gl_t conf_lib_hid_gl;
+
+#include "draw_INIT.h"
+
+/* return the first draw implementation that would work with current opengl */
+RND_INLINE const hidgl_draw_t *hidgl_find_draw(const RND_CFT_LIST *pref)
 {
-	stencilgl_init();
+	hidgl_draw_t *d;
+
+	hidgl_draw_init_();
+
+	if ((pref != NULL) && (rnd_conflist_length(pref) > 0)) {
+		/* order is coming from preference */
+		gdl_iterator_t it;
+		rnd_conf_listitem_t *p;
+
+		rnd_conflist_foreach(pref, &it, p) {
+			for(d = hidgl_draws; d != NULL; d = d->next)
+				if (strcmp(d->name, p->payload) == 0)
+					if (d->init() == 0) return d;
+		}
+	}
+	else {
+		/* fallback: try them one by one, in order, if there was no preference */
+		for(d = hidgl_draws; d != NULL; d = d->next)
+			if (d->init() == 0) return d;
+	}
+
+	/* final fallback: did not find anything working  */
+	if (hidgl_draw_error.init() == 0) return &hidgl_draw_error;
+	return NULL;
 }
 
+hidgl_draw_t hidgl_draw;
+hidgl_stenc_t hidgl_stenc;
+
+int hidgl_init(void)
+{
+	if (hidgl_draw.name == NULL) {
+		const hidgl_draw_t *drw = hidgl_find_draw(&conf_lib_hid_gl.plugins.lib_hid_gl.backend.preference);
+
+		if (drw == NULL)
+			return -1;
+
+		hidgl_draw = *drw;
+	}
+	return 0;
+}
+
+int hidgl_new_context(void)
+{
+	return hidgl_draw.new_context();
+}
+
+/* return the first stenc implementation that would work with current opengl */
+RND_INLINE const hidgl_stenc_t *hidgl_find_stenc(const RND_CFT_LIST *pref, int *stencil_bits)
+{
+	hidgl_stenc_t *d;
+
+	hidgl_stenc_init_();
+
+	if ((pref != NULL) && (rnd_conflist_length(pref) > 0)) {
+		/* order is coming from preference */
+		gdl_iterator_t it;
+		rnd_conf_listitem_t *p;
+
+		rnd_conflist_foreach(pref, &it, p) {
+			for(d = hidgl_stencs; d != NULL; d = d->next)
+				if (strcmp(d->name, p->payload) == 0)
+					if (d->init(stencil_bits) == 0) return d;
+		}
+	}
+	else {
+		/* fallback: try them one by one, in order, if there was no preference */
+		for(d = hidgl_stencs; d != NULL; d = d->next)
+			if (d->init(stencil_bits) == 0) return d;
+	}
+
+	/* final fallback: did not find anything working  */
+	if (hidgl_stenc_error.init(stencil_bits) == 0) return &hidgl_stenc_error;
+	return NULL;
+}
+
+int hidgl_stencil_init(void)
+{
+	if (hidgl_stenc.name == NULL) {
+		int stencil_bits = 0;
+		const hidgl_stenc_t *stc = hidgl_find_stenc(&conf_lib_hid_gl.plugins.lib_hid_gl.stencil.preference, &stencil_bits);
+
+		if (stc == NULL)
+			return -1;
+
+		hidgl_stenc = *stc;
+		return stencilgl_init(stencil_bits);
+	}
+	return 0;
+}
+
+void hidgl_uninit(void)
+{
+	if (hidgl_draw.uninit != NULL)
+		hidgl_draw.uninit();
+}
+
+void hidgl_flush_drawing(void)
+{
+	hidgl_draw.prim_flush();
+}
+
+void hidgl_flush(void)
+{
+	hidgl_draw.flush();
+}
+
+void hidgl_reset(void)
+{
+	hidgl_draw.reset();
+}
+
+void hidgl_set_color(float r, float g, float b, float a)
+{
+	hidgl_draw.set_color(r, g, b, a);
+}
+
+void hidgl_set_grid_color(float r, float g, float b)
+{
+	if (hidgl_draw.xor_inverts_clr)
+		hidgl_draw.set_color(1.0-r, 1.0-g, 1.0-b, 1);
+	else
+		hidgl_draw.set_color(r, g, b, 1);
+}
+
+void hidgl_set_view(double tx, double ty, double zx, double zy, double zz)
+{
+	hidgl_draw.set_view(tx, ty, zx, zy, zz);
+}
+
+long hidgl_texture_import(unsigned char *pixels, int width, int height, int has_alpha)
+{
+	return hidgl_draw.texture_import(pixels, width, height, has_alpha);
+}
+
+void hidgl_texture_free(long id)
+{
+	hidgl_draw.texture_free(id);
+}
+
+void hidgl_push_matrix(int projection)
+{
+	hidgl_draw.push_matrix(projection);
+}
+
+void hidgl_pop_matrix(int projection)
+{
+	hidgl_draw.pop_matrix(projection);
+}
+
+
 static rnd_composite_op_t composite_op = RND_HID_COMP_RESET;
 static rnd_bool direct_mode = rnd_true;
-static int comp_stencil_bit = 0;
 
 static GLfloat *grid_points = NULL, *grid_points3 = NULL;
 static int grid_point_capacity = 0, grid_point_capacity3 = 0;
 
-
-static inline void mode_reset(rnd_bool direct, const rnd_box_t *screen)
-{
-	drawgl_flush();
-	drawgl_reset();
-	glColorMask(0, 0, 0, 0); /* Disable colour drawing */
-	stencilgl_reset_stencil_usage();
-	glDisable(GL_COLOR_LOGIC_OP);
-	comp_stencil_bit = 0;
-}
-
-static inline void mode_positive(rnd_bool direct, const rnd_box_t *screen)
-{
-	if (comp_stencil_bit == 0)
-		comp_stencil_bit = stencilgl_allocate_clear_stencil_bit();
-	else
-		drawgl_flush();
-
-	glEnable(GL_STENCIL_TEST);
-	glDisable(GL_COLOR_LOGIC_OP);
-	stencilgl_mode_write_set(comp_stencil_bit);
-}
-
-static inline void mode_positive_xor(rnd_bool direct, const rnd_box_t *screen)
-{
-	drawgl_flush();
-	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-	glDisable(GL_STENCIL_TEST);
-	glEnable(GL_COLOR_LOGIC_OP);
-	glLogicOp(GL_XOR);
-}
-
-static inline void mode_negative(rnd_bool direct, const rnd_box_t *screen)
-{
-	glEnable(GL_STENCIL_TEST);
-	glDisable(GL_COLOR_LOGIC_OP);
-	
-	if (comp_stencil_bit == 0) {
-		/* The stencil isn't valid yet which means that this is the first pos/neg mode
-		 * set since the reset. The compositing stencil bit will be allocated. Because 
-		 * this is a negative mode and it's the first mode to be set, the stencil buffer
-		 * will be set to all ones.
-		 */
-		comp_stencil_bit = stencilgl_allocate_clear_stencil_bit();
-		stencilgl_mode_write_set(comp_stencil_bit);
-		drawgl_direct_draw_solid_rectangle(screen->X1, screen->Y1, screen->X2, screen->Y2);
-	}
-	else
-		drawgl_flush();
-
-	stencilgl_mode_write_clear(comp_stencil_bit);
-	drawgl_set_marker();
-}
-
-static inline void mode_flush(rnd_bool direct, rnd_bool xor_mode, const rnd_box_t *screen)
-{
-	drawgl_flush();
-	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-
-	if (comp_stencil_bit) {
-		glEnable(GL_STENCIL_TEST);
-
-		/* Setup the stencil to allow writes to the colour buffer if the 
-		 * comp_stencil_bit is set. After the operation, the comp_stencil_bit
-		 * will be cleared so that any further writes to this pixel are disabled.
-		 */
-		glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);
-		glStencilMask(comp_stencil_bit);
-		glStencilFunc(GL_EQUAL, comp_stencil_bit, comp_stencil_bit);
-
-		/* Draw all primtives through the stencil to the colour buffer. */
-		drawgl_draw_all(comp_stencil_bit);
-	}
-
-	glDisable(GL_STENCIL_TEST);
-	stencilgl_reset_stencil_usage();
-	comp_stencil_bit = 0;
-}
-
 rnd_composite_op_t hidgl_get_drawing_mode()
 {
 	return composite_op;
@@ -145,50 +222,38 @@ void hidgl_set_drawing_mode(rnd_hid_t *hid, rnd_composite_op_t op, rnd_bool dire
 	rnd_bool xor_mode = (composite_op == RND_HID_COMP_POSITIVE_XOR ? rnd_true : rnd_false);
 
 	/* If the previous mode was NEGATIVE then all of the primitives drawn
-	 * in that mode were used only for creating the stencil and will not be 
-	 * drawn directly to the colour buffer. Therefore these primitives can be 
-	 * discarded by rewinding the primitive buffer to the marker that was
-	 * set when entering NEGATIVE mode.
-	 */
+	   in that mode were used only for creating the stencil and will not be
+	   drawn directly to the color buffer. Therefore these primitives can be
+	   discarded by rewinding the primitive buffer to the marker that was
+	   set when entering NEGATIVE mode. */
 	if (composite_op == RND_HID_COMP_NEGATIVE) {
-		drawgl_flush();
-		drawgl_rewind_to_marker();
+		hidgl_draw.prim_flush();
+		hidgl_draw.prim_rewind_to_marker();
 	}
 
 	composite_op = op;
 	direct_mode = direct;
 
 	switch (op) {
-		case RND_HID_COMP_RESET:
-			mode_reset(direct, screen);
-			break;
-		case RND_HID_COMP_POSITIVE_XOR:
-			mode_positive_xor(direct, screen);
-			break;
-		case RND_HID_COMP_POSITIVE:
-			mode_positive(direct, screen);
-			break;
-		case RND_HID_COMP_NEGATIVE:
-			mode_negative(direct, screen);
-			break;
-		case RND_HID_COMP_FLUSH:
-			mode_flush(direct, xor_mode, screen);
-			break;
-		default:
-			break;
+		case RND_HID_COMP_RESET:         drawgl_mode_reset(direct, screen); break;
+		case RND_HID_COMP_POSITIVE_XOR:  drawgl_mode_positive_xor(direct, screen); break;
+		case RND_HID_COMP_POSITIVE:      drawgl_mode_positive(direct, screen); break;
+		case RND_HID_COMP_NEGATIVE:      drawgl_mode_negative(direct, screen); break;
+		case RND_HID_COMP_FLUSH:         drawgl_mode_flush(direct, xor_mode, screen); break;
+		default: break;
 	}
-
 }
 
 
 void hidgl_fill_rect(rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2)
 {
-	drawgl_add_triangle(x1, y1, x1, y2, x2, y2);
-	drawgl_add_triangle(x2, y1, x2, y2, x1, y1);
+	/* if this is done with hidgl_draw.prim_add_fillrect(), it breaks the mask */
+	hidgl_draw.prim_add_triangle(x1, y1, x1, y2, x2, y2);
+	hidgl_draw.prim_add_triangle(x2, y1, x2, y2, x1, y1);
 }
 
 
-static inline void reserve_grid_points(int n, int n3)
+RND_INLINE void reserve_grid_points(int n, int n3)
 {
 	if (n > grid_point_capacity) {
 		grid_point_capacity = n + 10;
@@ -200,7 +265,7 @@ static inline void reserve_grid_points(int n, int n3)
 	}
 }
 
-void hidgl_draw_local_grid(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_coord_t cy, int radius, double scale, rnd_bool cross_grid)
+void hidgl_draw_local_grid(rnd_hidlib_t *hidlib, rnd_coord_t grd, rnd_coord_t cx, rnd_coord_t cy, int radius, double scale, rnd_bool cross_grid)
 {
 	int npoints = 0;
 	rnd_coord_t x, y;
@@ -237,22 +302,24 @@ void hidgl_draw_local_grid(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_coord_t cy,
 		}
 	}
 
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glVertexPointer(2, GL_FLOAT, 0, grid_points);
-	glDrawArrays(GL_POINTS, 0, npoints);
-	glDisableClientState(GL_VERTEX_ARRAY);
-
+	hidgl_draw.xor_start();
+	hidgl_draw.draw_points_pre(grid_points);
+	hidgl_draw.draw_points(npoints);
+	hidgl_draw.draw_points_post();
+	hidgl_draw.xor_end();
 }
 
-void hidgl_draw_grid(rnd_hidlib_t *hidlib, rnd_box_t *drawn_area, double scale, rnd_bool cross_grid)
+void hidgl_draw_grid(rnd_hidlib_t *hidlib, rnd_coord_t grd, rnd_box_t *drawn_area, double scale, rnd_bool cross_grid)
 {
 	rnd_coord_t x1, y1, x2, y2, n, i, n3;
 	double x, y;
 
-	x1 = rnd_grid_fit(MAX(0, drawn_area->X1), hidlib->grid, hidlib->grid_ox);
-	y1 = rnd_grid_fit(MAX(0, drawn_area->Y1), hidlib->grid, hidlib->grid_oy);
-	x2 = rnd_grid_fit(MIN(hidlib->size_x, drawn_area->X2), hidlib->grid, hidlib->grid_ox);
-	y2 = rnd_grid_fit(MIN(hidlib->size_y, drawn_area->Y2), hidlib->grid, hidlib->grid_oy);
+	hidgl_draw.xor_start();
+
+	x1 = rnd_grid_fit(MAX(0, drawn_area->X1), grd, hidlib->grid_ox);
+	y1 = rnd_grid_fit(MAX(0, drawn_area->Y1), grd, hidlib->grid_oy);
+	x2 = rnd_grid_fit(MIN(hidlib->size_x, drawn_area->X2), grd, hidlib->grid_ox);
+	y2 = rnd_grid_fit(MIN(hidlib->size_y, drawn_area->Y2), grd, hidlib->grid_oy);
 
 	if (x1 > x2) {
 		rnd_coord_t tmp = x1;
@@ -266,52 +333,53 @@ void hidgl_draw_grid(rnd_hidlib_t *hidlib, rnd_box_t *drawn_area, double scale,
 		y2 = tmp;
 	}
 
-	n = (int)((x2 - x1) / hidlib->grid + 0.5) + 1;
+	n = (int)((x2 - x1) / grd + 0.5) + 1;
 	reserve_grid_points(n, cross_grid ? n*2 : 0);
 
 	/* draw grid center points and y offset points */
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glVertexPointer(2, GL_FLOAT, 0, grid_points);
+	hidgl_draw.draw_points_pre(grid_points);
 
 	n = 0;
-	for(x = x1; x <= x2; x += hidlib->grid, ++n)
+	for(x = x1; x <= x2; x += grd, ++n)
 		grid_points[2 * n + 0] = x;
 
-	for(y = y1; y <= y2; y += hidlib->grid) {
+	for(y = y1; y <= y2; y += grd) {
 		for(i = 0; i < n; i++)
 			grid_points[2 * i + 1] = y;
-		glDrawArrays(GL_POINTS, 0, n);
-		if (cross_grid) {
+		hidgl_draw.draw_points(n);
+
+		if (cross_grid) { /* vertical extension */
 			for(i = 0; i < n; i++)
 				grid_points[2 * i + 1] = y-scale;
-			glDrawArrays(GL_POINTS, 0, n);
+			hidgl_draw.draw_points(n);
 			for(i = 0; i < n; i++)
 				grid_points[2 * i + 1] = y+scale;
-			glDrawArrays(GL_POINTS, 0, n);
+			hidgl_draw.draw_points(n);
 		}
 	}
+	hidgl_draw.draw_points_post();
 
-	glDisableClientState(GL_VERTEX_ARRAY);
 
 	if (cross_grid) {
-		/* draw grid points around the crossings in x.x pattern */
-		glEnableClientState(GL_VERTEX_ARRAY);
-		glVertexPointer(2, GL_FLOAT, 0, grid_points3);
+		/* draw grid points around the crossings in x.x pattern (horizontal arms) */
+		hidgl_draw.draw_points_pre(grid_points3);
 
 		n3 = 0;
-		for(x = x1; x <= x2; x += hidlib->grid) {
+		for(x = x1; x <= x2; x += grd) {
 			grid_points3[2 * n3 + 0] = x - scale; n3++;
 			grid_points3[2 * n3 + 0] = x + scale; n3++;
 		}
 
-		for(y = y1; y <= y2; y += hidlib->grid) {
+		for(y = y1; y <= y2; y += grd) {
 			for(i = 0; i < n3; i++)
 				grid_points3[2 * i + 1] = y;
-			glDrawArrays(GL_POINTS, 0, n3);
+			hidgl_draw.draw_points(n3);
 		}
+
+		hidgl_draw.draw_points_post();
 	}
 
-	glDisableClientState(GL_VERTEX_ARRAY);
+	hidgl_draw.xor_end();
 }
 
 #define MAX_PIXELS_ARC_TO_CHORD 0.5
@@ -329,7 +397,7 @@ int calc_slices(float pix_radius, float sweep_angle)
 
 #define MIN_TRIANGLES_PER_CAP 3
 #define MAX_TRIANGLES_PER_CAP 90
-static void draw_cap(rnd_coord_t width, rnd_coord_t x, rnd_coord_t y, rnd_angle_t angle, double scale)
+static void draw_round_cap(rnd_coord_t width, rnd_coord_t x, rnd_coord_t y, rnd_angle_t angle, double scale)
 {
 	float last_capx, last_capy;
 	float capx, capy;
@@ -343,14 +411,14 @@ static void draw_cap(rnd_coord_t width, rnd_coord_t x, rnd_coord_t y, rnd_angle_
 	if (slices > MAX_TRIANGLES_PER_CAP)
 		slices = MAX_TRIANGLES_PER_CAP;
 
-	drawgl_reserve_triangles(slices);
+	hidgl_draw.prim_reserve_triangles(slices);
 
 	last_capx = radius * cosf(angle * M_PI / 180.) + x;
 	last_capy = -radius * sinf(angle * M_PI / 180.) + y;
 	for(i = 0; i < slices; i++) {
 		capx = radius * cosf(angle * M_PI / 180. + ((float)(i + 1)) * M_PI / (float)slices) + x;
 		capy = -radius * sinf(angle * M_PI / 180. + ((float)(i + 1)) * M_PI / (float)slices) + y;
-		drawgl_add_triangle(last_capx, last_capy, capx, capy, x, y);
+		hidgl_draw.prim_add_triangle(last_capx, last_capy, capx, capy, x, y);
 		last_capx = capx;
 		last_capy = capy;
 	}
@@ -363,11 +431,11 @@ void hidgl_draw_line(rnd_cap_style_t cap, rnd_coord_t width, rnd_coord_t x1, rnd
 	double angle;
 	float deltax, deltay, length;
 	float wdx, wdy;
-	int circular_caps = 0;
+	int round_caps = 0;
 	rnd_coord_t orig_width = width;
 
 	if ((width == 0) || (!NEEDS_CAP(orig_width, scale)))
-		drawgl_add_line(x1, y1, x2, y2);
+		hidgl_draw.prim_add_line(x1, y1, x2, y2);
 	else {
 		if (width < scale)
 			width = scale;
@@ -401,7 +469,7 @@ void hidgl_draw_line(rnd_cap_style_t cap, rnd_coord_t width, rnd_coord_t x1, rnd
 
 		switch (cap) {
 			case rnd_cap_round:
-				circular_caps = 1;
+				round_caps = 1;
 				break;
 
 			case rnd_cap_square:
@@ -413,15 +481,15 @@ void hidgl_draw_line(rnd_cap_style_t cap, rnd_coord_t width, rnd_coord_t x1, rnd
 
 			default:
 				assert(!"unhandled cap");
-				circular_caps = 1;
+				round_caps = 1;
 		}
 
-		drawgl_add_triangle(x1 - wdx, y1 - wdy, x2 - wdx, y2 - wdy, x2 + wdx, y2 + wdy);
-		drawgl_add_triangle(x1 - wdx, y1 - wdy, x2 + wdx, y2 + wdy, x1 + wdx, y1 + wdy);
+		hidgl_draw.prim_add_triangle(x1 - wdx, y1 - wdy, x2 - wdx, y2 - wdy, x2 + wdx, y2 + wdy);
+		hidgl_draw.prim_add_triangle(x1 - wdx, y1 - wdy, x2 + wdx, y2 + wdy, x1 + wdx, y1 + wdy);
 
-		if (circular_caps) {
-			draw_cap(width, x1, y1, angle, scale);
-			draw_cap(width, x2, y2, angle + 180., scale);
+		if (round_caps) {
+			draw_round_cap(width, x1, y1, angle, scale);
+			draw_round_cap(width, x2, y2, angle + 180., scale);
 		}
 	}
 }
@@ -472,7 +540,7 @@ void hidgl_draw_arc(rnd_coord_t width, rnd_coord_t x, rnd_coord_t y, rnd_coord_t
 	if (slices > MAX_SLICES_PER_ARC)
 		slices = MAX_SLICES_PER_ARC;
 
-	drawgl_reserve_triangles(2 * slices);
+	hidgl_draw.prim_reserve_triangles(2 * slices);
 
 	angle_incr_rad = delta_angle_rad / (float)slices;
 
@@ -490,8 +558,8 @@ void hidgl_draw_arc(rnd_coord_t width, rnd_coord_t x, rnd_coord_t y, rnd_coord_t
 		outer_x = -outer_r * cos_ang + x;
 		outer_y = outer_r * sin_ang + y;
 
-		drawgl_add_triangle(last_inner_x, last_inner_y, last_outer_x, last_outer_y, outer_x, outer_y);
-		drawgl_add_triangle(last_inner_x, last_inner_y, inner_x, inner_y, outer_x, outer_y);
+		hidgl_draw.prim_add_triangle(last_inner_x, last_inner_y, last_outer_x, last_outer_y, outer_x, outer_y);
+		hidgl_draw.prim_add_triangle(last_inner_x, last_inner_y, inner_x, inner_y, outer_x, outer_y);
 
 		last_inner_x = inner_x;
 		last_inner_y = inner_y;
@@ -504,19 +572,19 @@ void hidgl_draw_arc(rnd_coord_t width, rnd_coord_t x, rnd_coord_t y, rnd_coord_t
 		return;
 
 	if (NEEDS_CAP(orig_width, scale)) {
-		draw_cap(width, x + rx * -cosf(start_angle_rad), y + rx * sinf(start_angle_rad), start_angle, scale);
-		draw_cap(width, x + rx * -cosf(start_angle_rad + delta_angle_rad), y + rx * sinf(start_angle_rad + delta_angle_rad), start_angle + delta_angle + 180., scale);
+		draw_round_cap(width, x + rx * -cosf(start_angle_rad), y + rx * sinf(start_angle_rad), start_angle, scale);
+		draw_round_cap(width, x + rx * -cosf(start_angle_rad + delta_angle_rad), y + rx * sinf(start_angle_rad + delta_angle_rad), start_angle + delta_angle + 180., scale);
 	}
 }
 
 void hidgl_draw_rect(rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2)
 {
-	drawgl_add_rectangle(x1, y1, x2, y2);
+	hidgl_draw.prim_add_rect(x1, y1, x2, y2);
 }
 
 void hidgl_draw_texture_rect(rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2, unsigned long texture_id)
 {
-	drawgl_add_texture_quad(x1, y1, 0.0, 0.0, x2, y1, 1.0, 0.0, x2, y2, 1.0, 1.0, x1, y2, 0.0, 1.0, texture_id);
+	hidgl_draw.prim_add_textrect(x1, y1, 0.0, 0.0, x2, y1, 1.0, 0.0, x2, y2, 1.0, 1.0, x1, y2, 0.0, 1.0, texture_id);
 }
 
 void hidgl_fill_circle(rnd_coord_t vx, rnd_coord_t vy, rnd_coord_t vr, double scale)
@@ -539,7 +607,7 @@ void hidgl_fill_circle(rnd_coord_t vx, rnd_coord_t vy, rnd_coord_t vr, double sc
 	if (slices > MAX_TRIANGLES_PER_CIRCLE)
 		slices = MAX_TRIANGLES_PER_CIRCLE;
 
-	drawgl_reserve_triangles(slices);
+	hidgl_draw.prim_reserve_triangles(slices);
 
 	last_x = vx + vr;
 	last_y = vy;
@@ -548,7 +616,7 @@ void hidgl_fill_circle(rnd_coord_t vx, rnd_coord_t vy, rnd_coord_t vr, double sc
 		float x, y;
 		x = radius * cosf(((float)(i + 1)) * 2. * M_PI / (float)slices) + vx;
 		y = radius * sinf(((float)(i + 1)) * 2. * M_PI / (float)slices) + vy;
-		drawgl_add_triangle(vx, vy, last_x, last_y, x, y);
+		hidgl_draw.prim_add_triangle(vx, vy, last_x, last_y, x, y);
 		last_x = x;
 		last_y = y;
 	}
@@ -577,7 +645,7 @@ static void myFreeCombined()
 static void myCombine(GLdouble coords[3], void *vertex_data[4], GLfloat weight[4], void **dataOut)
 {
 #define MAX_COMBINED_VERTICES 2500
-	static GLdouble combined_vertices[3 *MAX_COMBINED_VERTICES];
+	static GLdouble combined_vertices[3 * MAX_COMBINED_VERTICES];
 	static int num_combined_vertices = 0;
 
 	GLdouble *new_vertex;
@@ -620,7 +688,7 @@ static void myVertex(GLdouble *vertex_data)
 			stashed_vertices++;
 		}
 		else {
-			drawgl_add_triangle(triangle_vertices[0], triangle_vertices[1], triangle_vertices[2], triangle_vertices[3], vertex_data[0], vertex_data[1]);
+			hidgl_draw.prim_add_triangle(triangle_vertices[0], triangle_vertices[1], triangle_vertices[2], triangle_vertices[3], vertex_data[0], vertex_data[1]);
 			if (tessVertexType == GL_TRIANGLE_STRIP) {
 				/* STRIP saves the last two vertices for re-use in the next triangle */
 				triangle_vertices[0] = triangle_vertices[2];
@@ -636,7 +704,7 @@ static void myVertex(GLdouble *vertex_data)
 		triangle_vertices[triangle_comp_idx++] = vertex_data[1];
 		stashed_vertices++;
 		if (stashed_vertices == 3) {
-			drawgl_add_triangle(triangle_vertices[0], triangle_vertices[1], triangle_vertices[2], triangle_vertices[3], triangle_vertices[4], triangle_vertices[5]);
+			hidgl_draw.prim_add_triangle(triangle_vertices[0], triangle_vertices[1], triangle_vertices[2], triangle_vertices[3], triangle_vertices[4], triangle_vertices[5]);
 			triangle_comp_idx = 0;
 			stashed_vertices = 0;
 		}
@@ -645,7 +713,7 @@ static void myVertex(GLdouble *vertex_data)
 		printf("Vertex received with unknown type\n");
 }
 
-/* Intentaional code duplication for performance */
+/* Intentional code duplication for performance */
 void hidgl_fill_polygon(int n_coords, rnd_coord_t *x, rnd_coord_t *y)
 {
 	int i;
@@ -680,7 +748,7 @@ void hidgl_fill_polygon(int n_coords, rnd_coord_t *x, rnd_coord_t *y)
 	free(vertices);
 }
 
-/* Intentaional code duplication for performance */
+/* Intentional code duplication for performance */
 void hidgl_fill_polygon_offs(int n_coords, rnd_coord_t *x, rnd_coord_t *y, rnd_coord_t dx, rnd_coord_t dy)
 {
 	int i;
@@ -717,42 +785,12 @@ void hidgl_fill_polygon_offs(int n_coords, rnd_coord_t *x, rnd_coord_t *y, rnd_c
 
 void hidgl_expose_init(int w, int h, const rnd_color_t *bg_c)
 {
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-
-	glViewport(0, 0, w, h);
-
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	glOrtho(0, w, h, 0, 0, 100);
-	glMatrixMode(GL_MODELVIEW);
-	glLoadIdentity();
-	glTranslatef(0.0f, 0.0f, -HIDGL_Z_NEAR);
-
-	glEnable(GL_STENCIL_TEST);
-	glClearColor(bg_c->fr, bg_c->fg, bg_c->fb, 1.);
-	glStencilMask(~0);
-	glClearStencil(0);
-	glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
-	stencilgl_reset_stencil_usage();
-
-	/* Disable the stencil test until we need it - otherwise it gets dirty */
-	glDisable(GL_STENCIL_TEST);
-	glStencilMask(0);
-	glStencilFunc(GL_ALWAYS, 0, 0);
+	hidgl_draw.expose_init(w, h, bg_c);
 }
 
 void hidgl_draw_crosshair(rnd_coord_t x, rnd_coord_t y, float red, float green, float blue, rnd_coord_t minx, rnd_coord_t miny, rnd_coord_t maxx, rnd_coord_t maxy)
 {
-	int i;
-	float points[4][6];
-
-	for(i=0; i<4; ++i) {
-		points[i][2] = red;
-		points[i][3] = green;
-		points[i][4] = blue;
-		points[i][5] = 1.0f;
-	}
+	GLfloat points[4][6];
 
 	points[0][0] = x;
 	points[0][1] = miny;
@@ -763,44 +801,21 @@ void hidgl_draw_crosshair(rnd_coord_t x, rnd_coord_t y, float red, float green,
 	points[3][0] = maxx;
 	points[3][1] = y;
 
-	glEnable(GL_COLOR_LOGIC_OP);
-	glLogicOp(GL_XOR);
-
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
-	glVertexPointer(2, GL_FLOAT, sizeof(float) * 6, points);
-	glColorPointer(4, GL_FLOAT, sizeof(float) * 6, &points[0][2]);
-	glDrawArrays(GL_LINES, 0, 4);
-	glDisableClientState(GL_VERTEX_ARRAY);
-	glDisableClientState(GL_COLOR_ARRAY);
+	hidgl_draw.xor_start();
+	hidgl_draw.draw_lines6((GLfloat *)points, 4, red, green, blue, 1.0);
+	hidgl_draw.xor_end();
 }
 
-void hidgl_draw_solid_rect(rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2, float red, float green, float blue)
+void hidgl_draw_initial_fill(rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2, float r, float g, float b)
 {
-	float points[4][6];
-	int i;
+	hidgl_draw.xor_end();
 
-	for(i=0; i<4; ++i) {
-		points[i][2] = red;
-		points[i][3] = green;
-		points[i][4] = blue;
-		points[i][5] = 1.0f;
-	}
+	/* we can cheat here: this is called only once, before other drawing commands
+	   to fill the background. */
+	hidgl_draw.set_color(r, g, b, 1.0f);
 
-	points[0][0] = x1;
-	points[0][1] = y1;
-	points[1][0] = x2;
-	points[1][1] = y1;
-	points[2][0] = x2;
-	points[2][1] = y2;
-	points[3][0] = x1;
-	points[3][1] = y2;
-
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
-	glVertexPointer(2, GL_FLOAT, sizeof(float) * 6, points);
-	glColorPointer(4, GL_FLOAT, sizeof(float) * 6, &points[0][2]);
-	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
-	glDisableClientState(GL_VERTEX_ARRAY);
-	glDisableClientState(GL_COLOR_ARRAY);
+	/* fillrect breaks for some reason, drawing two triangles is more stable */
+	hidgl_draw.prim_add_triangle(x1, y1, x1, y2, x2, y2);
+	hidgl_draw.prim_add_triangle(x2, y1, x2, y2, x1, y1);
 }
+
